@@ -41,6 +41,7 @@
 (define dynamic-iters 1)
 (define earley-iters 1)
 (define fibc-iters 1)
+(define fftrad4-iters 1)
 (define graphs-iters 1)
 (define lattice-iters 1)
 (define matrix-iters 1)
@@ -60,3 +61,6 @@
 (define sboyer-iters 1)
 (define gcbench-iters 1)
 (define compiler-iters 1)
+(define chud100K-iters 1)
+(define chud1K-iters 1)
+(define pi10K-iters 1)
