@@ -2,7 +2,7 @@
 
 ;;; File: "_gsclib.scm"
 
-;;; Copyright (c) 1994-2015 by Marc Feeley, All Rights Reserved.
+;;; Copyright (c) 1994-2018 by Marc Feeley, All Rights Reserved.
 
 (include "generic.scm")
 
@@ -36,9 +36,11 @@
          #!key
          (options (macro-absent-obj))
          (output (macro-absent-obj))
-         (module-name (macro-absent-obj)))
+         (module-name (macro-absent-obj))
+         (linker-name (macro-absent-obj))
+         (expression (macro-absent-obj)))
   (macro-force-vars (filename)
-    (macro-check-string filename 1 (compile-file-to-target filename . other);;;;;;
+    (macro-check-string filename 1 (compile-file-to-target filename . other) ;;;;;;
       (let* ((opts
               (if (##eq? options (macro-absent-obj))
                   '()
@@ -54,22 +56,42 @@
               (if (##eq? module-name (macro-absent-obj))
                   #f
                   (macro-force-vars (module-name)
-                    module-name))))
+                    module-name)))
+             (link-name
+              (if (##eq? linker-name (macro-absent-obj))
+                  #f
+                  (macro-force-vars (linker-name)
+                    linker-name)))
+             (filename-or-source
+              (if (##eq? expression (macro-absent-obj))
+                  filename
+                  (if (##source? expression)
+                      expression
+                      (##make-source
+                       expression
+                       (##make-locat (##path->container filename) 0))))))
         (cond ((##not (or (##null? opts)
                           (##pair? opts)))
-               (error "list expected for options: parameter"));;;;;;;
+               (error "list expected for options: parameter")) ;;;;;;;
               ((##not (##string? out))
-               (error "string expected for output: parameter"));;;;;;;;;;
+               (error "string expected for output: parameter")) ;;;;;;;;;;
               ((##not (or (##not mod-name) (##string? mod-name)))
-               (error "string or #f expected for module-name: parameter"));;;;;;;;;;
+               (error "string or #f expected for module-name: parameter")) ;;;;;;;;;;
+              ((##not (or (##not link-name) (##string? link-name)))
+               (error "string or #f expected for linker-name: parameter")) ;;;;;;;;;;
               (else
-               (##compile-file-to-target filename
+               (##compile-file-to-target filename-or-source
                                          opts
                                          out
-                                         mod-name)))))))
+                                         mod-name
+                                         link-name)))))))
 
-(define (##compile-file-to-target filename options output mod-name)
-  (let* ((options
+(define (##compile-file-to-target filename-or-source options output mod-name link-name)
+  (let* ((filename
+          (if (##source? filename-or-source)
+              (##source-path filename-or-source)
+              filename-or-source))
+         (options
           (##compile-options-normalize options))
          (expanded-output
           (##path-normalize output))
@@ -93,12 +115,15 @@
                (##path-strip-extension
                 (if output-directory?
                     filename
-                    expanded-output))))))
-    (c#cf filename
+                    expanded-output)))))
+         (linker-name
+          (or link-name
+              module-name)))
+    (c#cf filename-or-source
           options
           output-filename-gen
           module-name
-          module-name)))
+          linker-name)))
 
 (define (compile-file
          filename
@@ -106,11 +131,14 @@
          #!key
          (options (macro-absent-obj))
          (output (macro-absent-obj))
+         (module-name (macro-absent-obj))
+         (linker-name (macro-absent-obj))
          (cc-options (macro-absent-obj))
          (ld-options-prelude (macro-absent-obj))
-         (ld-options (macro-absent-obj)))
+         (ld-options (macro-absent-obj))
+         (expression (macro-absent-obj)))
   (macro-force-vars (filename)
-    (macro-check-string filename 1 (compile-file filename . other);;;;;;
+    (macro-check-string filename 1 (compile-file filename . other) ;;;;;;
       (let* ((opts
               (if (##eq? options (macro-absent-obj))
                   '()
@@ -122,6 +150,16 @@
                    (##path-normalize filename))
                   (macro-force-vars (output)
                     output)))
+             (mod-name
+              (if (##eq? module-name (macro-absent-obj))
+                  #f
+                  (macro-force-vars (module-name)
+                    module-name)))
+             (link-name
+              (if (##eq? linker-name (macro-absent-obj))
+                  #f
+                  (macro-force-vars (linker-name)
+                    linker-name)))
              (cc-opts
               (if (##eq? cc-options (macro-absent-obj))
                   ""
@@ -136,35 +174,53 @@
               (if (##eq? ld-options (macro-absent-obj))
                   ""
                   (macro-force-vars (ld-options)
-                    ld-options))))
+                    ld-options)))
+             (filename-or-source
+              (if (##eq? expression (macro-absent-obj))
+                  filename
+                  (if (##source? expression)
+                      expression
+                      (##make-source
+                       expression
+                       (##make-locat (##path->container filename) 0))))))
         (cond ((##not (or (##null? opts)
                           (##pair? opts)))
-               (error "list expected for options: parameter"));;;;;;;
+               (error "list expected for options: parameter")) ;;;;;;;
               ((##not (##string? out))
-               (error "string expected for output: parameter"));;;;;;;;;;
+               (error "string expected for output: parameter")) ;;;;;;;;;;
+              ((##not (or (##not mod-name) (##string? mod-name)))
+               (error "string or #f expected for module-name: parameter")) ;;;;;;;;;;
+              ((##not (or (##not link-name) (##string? link-name)))
+               (error "string or #f expected for linker-name: parameter")) ;;;;;;;;;;
               ((##not (##string? cc-opts))
-               (error "string expected for cc-options: parameter"));;;;;;;;;;
+               (error "string expected for cc-options: parameter")) ;;;;;;;;;;
               ((##not (##string? ld-opts-prelude))
-               (error "string expected for ld-options-prelude: parameter"));;;;;;;;;;
+               (error "string expected for ld-options-prelude: parameter")) ;;;;;;;;;;
               ((##not (##string? ld-opts))
-               (error "string expected for ld-options: parameter"));;;;;;;;;;
+               (error "string expected for ld-options: parameter")) ;;;;;;;;;;
               (else
-               (##compile-file filename
+               (##compile-file filename-or-source
                                opts
                                out
+                               mod-name
+                               link-name
                                cc-opts
                                ld-opts-prelude
                                ld-opts)))))))
 
 (define (##compile-file
-         filename
+         filename-or-source
          options
          output
+         module-name
+         linker-name
          cc-options
          ld-options-prelude
          ld-options)
-  (let ((options
-         (##compile-options-normalize options)))
+  (let* ((options
+          (##compile-options-normalize options))
+         (target
+          (##extract-target options)))
 
     (define type
       (cond ((##assq 'obj options)
@@ -182,24 +238,30 @@
               (loop (##fx+ version 1))
               root-with-ext))))
 
-    (define (generate-output-filename root input-is-c-file?)
+    (define (generate-output-filename root input-is-target-file?)
       (case type
         ((obj)
-         (##string-append root ##os-obj-extension-string-saved))
+         (##string-append
+          root
+          ##os-obj-extension-string-saved))
         (else
-         (if input-is-c-file?
+         (if input-is-target-file?
              root
              (generate-next-version-of-object-file root)))))
 
-    (let* ((input-is-c-file?
+    (let* ((filename
+            (if (##source? filename-or-source)
+                (##source-path filename-or-source)
+                filename-or-source))
+           (input-is-target-file?
             (##assoc (##path-extension filename)
-                     (c#target-file-extensions (c#target-get 'C))))
-           (c-filename
-            (if input-is-c-file?
+                     (c#target-file-extensions target)))
+           (target-filename
+            (if input-is-target-file?
                 filename
                 (##string-append
                  (##path-strip-extension filename)
-                 (##caar (c#target-file-extensions (c#target-get 'C))))))
+                 (##caar (c#target-file-extensions target)))))
            (expanded-output
             (##path-normalize output))
            (output-directory?
@@ -213,42 +275,44 @@
                   (##path-strip-directory
                    (##path-strip-extension filename))
                   expanded-output)
-                 input-is-c-file?)
+                 input-is-target-file?)
                 expanded-output))
            (output-dir
             (##path-directory output-filename))
            (output-filename-no-dir
             (##path-strip-directory output-filename))
            (module-name
-            (##path-strip-extension output-filename-no-dir))
-           (unique-name
-            (if (##eq? type 'dyn)
-                output-filename-no-dir
-                module-name)))
-      (and (or input-is-c-file?
-               (c#cf filename
+            (or module-name
+                (##path-strip-extension output-filename-no-dir)))
+           (linker-name
+            (or linker-name
+                (if (##eq? type 'dyn)
+                    output-filename-no-dir
+                    module-name))))
+      (and (or input-is-target-file?
+               (c#cf filename-or-source
                      options
-                     (lambda () c-filename)
+                     (lambda () target-filename)
                      module-name
-                     unique-name))
+                     linker-name))
            (let ((exit-status
                   (##gambcomp
-                   'C
+                   (c#target-name target)
                    type
                    output-dir
-                   (##list c-filename)
+                   (##list target-filename)
                    output-filename-no-dir
                    (##assq 'verbose options)
                    (##list (##cons "CC_OPTIONS" cc-options)
                            (##cons "LD_OPTIONS_PRELUDE" ld-options-prelude)
                            (##cons "LD_OPTIONS" ld-options)))))
              (if (and (##not (##assq 'keep-c options))
-                      (##not (##string=? filename c-filename)))
-                 (##delete-file c-filename))
+                      (##not (##string=? filename target-filename)))
+                 (##delete-file target-filename))
              (if (##fx= exit-status 0)
                  output-filename
                  (##raise-error-exception
-                  "C compilation or link failed while compiling"
+                  "target compilation or link failed while compiling"
                   (##list filename))))))))
 
 (define (##build-executable
@@ -260,13 +324,15 @@
          ld-options)
   (let* ((options
           (##compile-options-normalize options))
+         (target
+          (##extract-target options))
          (output-dir
           (##path-directory output-filename))
          (output-filename-no-dir
           (##path-strip-directory output-filename))
          (exit-status
           (##gambcomp
-           'C
+           (c#target-name target)
            'exe
            output-dir
            obj-files
@@ -278,7 +344,7 @@
     (if (##fx= exit-status 0)
         output-filename
         (##raise-error-exception
-         "C link failed while linking"
+         "target link failed while linking"
          obj-files))))
 
 (define (##gambcomp
@@ -378,11 +444,22 @@
              stdout-redirection: #f
              stderr-redirection: #f))))
 
+(define (##extract-target options)
+  (let ((t (##assq 'target options)))
+    (c#with-exception-handling
+     (lambda ()
+       (c#target-get
+        (if (and (##pair? t)
+                 (##pair? (##cdr t)))
+            (##cadr t)
+            (c#default-target)))))))
+
 (define (link-incremental
          modules
          #!rest other;;;;;;;;;;
          #!key
          (output (macro-absent-obj))
+         (linker-name (macro-absent-obj))
          (base (macro-absent-obj))
          (warnings? (macro-absent-obj)))
   (macro-force-vars (modules)
@@ -405,6 +482,11 @@
                          (##path-normalize (##car (##car rev-mods))))
                         (macro-force-vars (output)
                           output)))
+                   (link-name
+                    (if (##eq? linker-name (macro-absent-obj))
+                        #f
+                        (macro-force-vars (linker-name)
+                          linker-name)))
                    (baselib
                     (if (##eq? base (macro-absent-obj))
                         (let ((gambitdir-lib
@@ -422,18 +504,22 @@
                           warnings?))))
               (cond ((##not (##string? out))
                      (error "string expected for output: parameter")) ;;;;;;;;;;
+                    ((##not (or (##not link-name) (##string? link-name)))
+                     (error "string or #f expected for linker-name: parameter"));;;;;;;;;;
                     ((##not (##string? baselib))
                      (error "string expected for base: parameter")) ;;;;;;;;;;
                     (else
                      (##link-incremental rev-mods
                                          out
+                                         link-name
                                          baselib
                                          warn?)))))))))
 
-(define (##link-incremental rev-mods output base warnings?)
+(define (##link-incremental rev-mods output linker-name base warnings?)
   (c#link-modules #t
                   (##cons (##list base) (##reverse rev-mods))
                   output
+                  linker-name
                   warnings?))
 
 (define (link-flat
@@ -441,6 +527,7 @@
          #!rest other;;;;;;;;;;
          #!key
          (output (macro-absent-obj))
+         (linker-name (macro-absent-obj))
          (warnings? (macro-absent-obj)))
   (macro-force-vars (modules)
     (let loop ((lst modules) (rev-mods '()))
@@ -462,6 +549,11 @@
                          (##path-normalize (##car (##car rev-mods))))
                         (macro-force-vars (output)
                           output)))
+                   (link-name
+                    (if (##eq? linker-name (macro-absent-obj))
+                        #f
+                        (macro-force-vars (linker-name)
+                          linker-name)))
                    (warn?
                     (if (##eq? warnings? (macro-absent-obj))
                         #t
@@ -469,15 +561,19 @@
                           warnings?))))
               (cond ((##not (##string? out))
                      (error "string expected for output: parameter")) ;;;;;;;;;;
+                    ((##not (or (##not link-name) (##string? link-name)))
+                     (error "string or #f expected for linker-name: parameter"));;;;;;;;;;
                     (else
                      (##link-flat rev-mods
                                   out
+                                  link-name
                                   warn?)))))))))
 
-(define (##link-flat rev-mods output warnings?)
+(define (##link-flat rev-mods output linker-name warnings?)
   (c#link-modules #f
                   (##reverse rev-mods)
                   output
+                  linker-name
                   warnings?))
 
 (define (##c-code . args) ;; avoid errors when using -expansion
