@@ -2,7 +2,7 @@
 
 ;;; File: "_host.scm"
 
-;;; Copyright (c) 1994-2011 by Marc Feeley, All Rights Reserved.
+;;; Copyright (c) 1994-2018 by Marc Feeley, All Rights Reserved.
 
 ;;;============================================================================
 
@@ -43,6 +43,9 @@
          (and (< 1 len)
               (char=? (string-ref str (- len 1)) #\:)))))
 
+(define (keyword-object-interned? key)
+  #t)
+
 ;; These definitions are needed to support objects which are not
 ;; standard in all implementations of Scheme.  On implementations which
 ;; do not support these objects, they are represented with symbols.
@@ -138,6 +141,9 @@
 ;;       (not (body-object? obj))
        ))
 
+(define (symbol-object-interned? sym)
+  #t)
+
 (define box-tag (list 'box))
 
 (define (box-object? obj)
@@ -166,6 +172,12 @@
 (define (open-input-file* path)
   (open-input-file path))
 
+(define (open-input-file*-preserving-case path)
+  (open-input-file* path))
+
+(define (open-output-file-preserving-case path)
+  (open-output-file path))
+
 ;; 'pp-expression' is used to pretty print an expression on a given
 ;; port.
 
@@ -572,7 +584,8 @@
 (define (max-fixnum32-div-max-lines)  8191)
 (define (subtype-structure) #f)
 
-(define (symbol-hash sym) 0)
+(define (symbol-object-hash sym) 0)
+(define (keyword-object-hash key) 0)
 
 ;; Tables.
 
@@ -634,6 +647,9 @@
 (define (keyword-object? obj)
   (##keyword? obj))
 
+(define (keyword-object-interned? key)
+  (##keyword-interned? key))
+
 (define false-object #f)
 
 (define (false-object? obj)
@@ -693,6 +709,9 @@
 (define (symbol-object? obj)
   (symbol? obj))
 
+(define (symbol-object-interned? sym)
+  (##symbol-interned? sym))
+
 (define (box-object? obj)
   (##box? obj))
 
@@ -719,8 +738,26 @@
    open-input-file
    path))
 
+(define (open-input-file*-preserving-case path)
+  (parameterize ((current-readtable
+                  (readtable-keywords-allowed?-set
+                   (readtable-case-conversion?-set
+                    (##make-standard-readtable)
+                    #f)
+                   #t)))
+    (open-input-file* path)))
+
 (define open-output-file ##open-output-file)
 
+(define (open-output-file-preserving-case path)
+  (parameterize ((current-readtable
+                  (readtable-keywords-allowed?-set
+                   (readtable-case-conversion?-set
+                    (##make-standard-readtable)
+                    #f)
+                   #t)))
+    (open-output-file path)))
+
 (define (pp-expression expr port)
   (pp expr port))
 
@@ -872,7 +909,10 @@
 (define (subtype-structure)
   (macro-subtype-structure))
 
-(define (symbol-hash sym)
-  (macro-symbol-hash sym))
+(define (symbol-object-hash sym)
+  (##symbol-hash sym))
+
+(define (keyword-object-hash key)
+  (##keyword-hash key))
 
 ;"
