@@ -2,7 +2,7 @@
 
 ;;; File: "_parms.scm"
 
-;;; Copyright (c) 1994-2016 by Marc Feeley, All Rights Reserved.
+;;; Copyright (c) 1994-2019 by Marc Feeley, All Rights Reserved.
 
 (include "fixnum.scm")
 
@@ -69,6 +69,7 @@
 (define unquote-sym            (string->canonical-symbol "unquote"))
 (define unquote-splicing-sym   (string->canonical-symbol "unquote-splicing"))
 
+(define **identity-sym         (string->canonical-symbol "##identity"))
 (define **not-sym              (string->canonical-symbol "##not"))
 (define **eq?-sym              (string->canonical-symbol "##eq?"))
 (define **eqv?-sym             (string->canonical-symbol "##eqv?"))
@@ -81,7 +82,7 @@
 (define **box-sym              (string->canonical-symbol "##box"))
 (define **unbox-sym            (string->canonical-symbol "##unbox"))
 (define **set-box!-sym         (string->canonical-symbol "##set-box!"))
-(define **make-promise-sym     (string->canonical-symbol "##make-promise"))
+(define **make-delay-promise-sym (string->canonical-symbol "##make-delay-promise"))
 
 (define ieee-scheme-sym        (string->canonical-symbol "ieee-scheme"))
 (define r4rs-scheme-sym        (string->canonical-symbol "r4rs-scheme"))
@@ -111,6 +112,7 @@
 (define warnings-sym           (string->canonical-symbol "warnings"))
 
 (define interrupts-enabled-sym (string->canonical-symbol "interrupts-enabled"))
+(define poll-on-return-sym     (string->canonical-symbol "poll-on-return"))
 
 (define debug-sym              (string->canonical-symbol "debug"))
 (define debug-location-sym     (string->canonical-symbol "debug-location"))
@@ -202,10 +204,7 @@
 
 ;;- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 
-;; Note: Module prefix and C id prefix must match the definitions
-;; in the "gambit.h" header file.
-
-(define module-prefix " ") ;; Identifier prefix for module names.
+;; Note: the C id prefix must match the definition in the "gambit.h" header file.
 
 (define c-id-prefix "___") ;; Identifier prefix for C identifiers generated by
                            ;; the C-interface and the C back-end.
@@ -553,7 +552,7 @@
 
 ;; Gambit system version numbers.
 
-(define (compiler-version) 408008) ;; 100000*major + 1000*minor + revision
+(define (compiler-version) 409003) ;; 100000*major + 1000*minor + revision
 
 (define compiler-version-string-prefix "v")
 (define compiler-version-string-suffix "")
