@@ -2,7 +2,7 @@
 
 ;;; File: "_ptree1.scm"
 
-;;; Copyright (c) 1994-2016 by Marc Feeley, All Rights Reserved.
+;;; Copyright (c) 1994-2019 by Marc Feeley, All Rights Reserved.
 
 (include "fixnum.scm")
 
@@ -479,6 +479,7 @@
 (define-boolean-decl warnings-sym)
 
 (define-boolean-decl interrupts-enabled-sym)
+(define-boolean-decl poll-on-return-sym)
 
 (define-boolean-decl debug-sym)
 (define-boolean-decl debug-location-sym)
@@ -514,7 +515,7 @@
   (env-declare env (list inline-primitives-sym #f)))
 
 (define (inlining-limit env) ; returns the inlining limit
-  (max 0 (min 1000000 (declaration-value inlining-limit-sym #f 350 env))))
+  (max 0 (min 1000000 (declaration-value inlining-limit-sym #f 370 env))))
 
 (define (block-compilation? env) ; true iff block compilation strategy
   (eq? (declaration-value 'compilation-strategy #f separate-sym env)
@@ -553,6 +554,9 @@
 (define (add-not-interrupts-enabled env)
   (env-declare env (list interrupts-enabled-sym #f)))
 
+(define (poll-on-return? env) ; true when interrupt checks should be generated on procedure returns
+  (declaration-value poll-on-return-sym #f #f env))
+
 (define (debug? env) ; true iff debugging information should be generated
   (declaration-value debug-sym #f compiler-option-debug env))
 
@@ -1846,7 +1850,7 @@
 (define (pt-delay source env use)
   (let ((code (source-code source)))
     (new-call* source (add-not-safe env)
-      (new-ref-extended-bindings source **make-promise-sym env)
+      (new-ref-extended-bindings source **make-delay-promise-sym env)
       (list (new-prc source env #f #f '() '() #f #f
               (pt (cadr code) env 'true))))))
 
@@ -1876,6 +1880,17 @@
       (key-object? code)
       (rest-object? code)
 ;;      (body-object? code)
+      (vector-object? code)
+      (u8vect? code)
+      (s8vect? code)
+      (u16vect? code)
+      (s16vect? code)
+      (u32vect? code)
+      (s32vect? code)
+      (u64vect? code)
+      (s64vect? code)
+      (f32vect? code)
+      (f64vect? code)
       ))
 
 (define (**quote-expr? source)
