@@ -220,6 +220,9 @@
 (define-macro (^set obj name val)
   `(univ-emit-set ctx ,obj ,name ,val))
 
+(define-macro (^attribute-exists? obj name)
+  `(univ-emit-attribute-exists? ctx ,obj ,name))
+
 (define-macro (^obj obj)
   `(univ-emit-obj ctx ,obj))
 
@@ -238,6 +241,9 @@
 (define-macro (^call-prim expr . params)
   `(univ-emit-call-prim ctx ,expr ,@params))
 
+(define-macro (^call-member expr fct . params)
+  `(univ-emit-call-member ctx ,expr ,fct ,@params))
+
 (define-macro (^jump expr . params)
   `(univ-emit-jump ctx ,expr ,@params))
 
@@ -447,6 +453,12 @@
 (define-macro (^absent)
   `(univ-emit-absent ctx))
 
+(define-macro (^deleted)
+  `(univ-emit-deleted ctx))
+
+(define-macro (^unused)
+  `(univ-emit-unused ctx))
+
 (define-macro (^unbound1)
   `(univ-emit-unbound1 ctx))
 
@@ -513,6 +525,9 @@
 (define-macro (^int val)
   `(univ-emit-int ctx ,val))
 
+(define-macro (^num-of-type type val)
+  `(univ-emit-num-of-type ctx ,type ,val))
+
 (define-macro (^fixnum-box val)
   `(univ-emit-fixnum-box ctx ,val))
 
@@ -538,6 +553,12 @@
 (define-macro (^dict-set expr1 expr2 expr3)
   `(univ-emit-dict-set ctx ,expr1 ,expr2 ,expr3))
 
+(define-macro (^dict-delete expr1 expr2)
+  `(univ-emit-dict-delete ctx ,expr1 ,expr2))
+
+(define-macro (^dict-length expr)
+  `(univ-emit-dict-length ctx ,expr))
+
 (define-macro (^member expr name)
   `(univ-emit-member ctx ,expr ,name))
 
@@ -568,6 +589,9 @@
 (define-macro (^float-toint val)
   `(univ-emit-float-toint ctx ,val))
 
+(define-macro (^float-math fn . params)
+  `(univ-emit-float-math ctx ,fn ,@params))
+
 (define-macro (^float-abs val)
   `(univ-emit-float-abs ctx ,val))
 
@@ -595,9 +619,15 @@
 (define-macro (^float-exp val)
   `(univ-emit-float-exp ctx ,val))
 
+(define-macro (^float-expm1 val)
+  `(univ-emit-float-expm1 ctx ,val))
+
 (define-macro (^float-log val)
   `(univ-emit-float-log ctx ,val))
 
+(define-macro (^float-log1p val)
+  `(univ-emit-float-log1p ctx ,val))
+
 (define-macro (^float-sin val)
   `(univ-emit-float-sin ctx ,val))
 
@@ -619,12 +649,36 @@
 (define-macro (^float-atan2 val1 val2)
   `(univ-emit-float-atan2 ctx ,val1 ,val2))
 
+(define-macro (^float-sinh val)
+  `(univ-emit-float-sinh ctx ,val))
+
+(define-macro (^float-cosh val)
+  `(univ-emit-float-cosh ctx ,val))
+
+(define-macro (^float-tanh val)
+  `(univ-emit-float-tanh ctx ,val))
+
+(define-macro (^float-asinh val)
+  `(univ-emit-float-asinh ctx ,val))
+
+(define-macro (^float-acosh val)
+  `(univ-emit-float-acosh ctx ,val))
+
+(define-macro (^float-atanh val)
+  `(univ-emit-float-atanh ctx ,val))
+
 (define-macro (^float-expt val1 val2)
   `(univ-emit-float-expt ctx ,val1 ,val2))
 
 (define-macro (^float-sqrt val)
   `(univ-emit-float-sqrt ctx ,val))
 
+(define-macro (^float-scalbn val1 val2)
+  `(univ-emit-float-scalbn ctx ,val1 ,val2))
+
+(define-macro (^float-ilogb val)
+  `(univ-emit-float-ilogb ctx ,val))
+
 (define-macro (^float-integer? val)
   `(univ-emit-float-integer? ctx ,val))
 
@@ -637,6 +691,12 @@
 (define-macro (^float-nan? val)
   `(univ-emit-float-nan? ctx ,val))
 
+(define-macro (^float-copysign val1 val2)
+  `(univ-emit-float-copysign ctx ,val1 ,val2))
+
+(define-macro (^float-eqv? val1 val2)
+  `(univ-emit-float-eqv? ctx ,val1 ,val2))
+
 (define-macro (^flonum-box val)
   `(univ-emit-flonum-box ctx ,val))
 
@@ -667,6 +727,30 @@
 (define-macro (^bignum-digits val)
   `(univ-emit-bignum-digits ctx ,val))
 
+(define-macro (^u32-box val)
+  `(univ-emit-u32-box ctx ,val))
+
+(define-macro (^u32-unbox u32)
+  `(univ-emit-u32-unbox ctx ,u32))
+
+(define-macro (^s32-box val)
+  `(univ-emit-s32-box ctx ,val))
+
+(define-macro (^s32-unbox s32)
+  `(univ-emit-s32-unbox ctx ,s32))
+
+(define-macro (^u64-box val)
+  `(univ-emit-u64-box ctx ,val))
+
+(define-macro (^u64-unbox u64)
+  `(univ-emit-u64-unbox ctx ,u64))
+
+(define-macro (^s64-box val)
+  `(univ-emit-s64-box ctx ,val))
+
+(define-macro (^s64-unbox s64)
+  `(univ-emit-s64-unbox ctx ,s64))
+
 (define-macro (^box? val)
   `(univ-emit-box? ctx ,val))
 
@@ -697,6 +781,15 @@
 (define-macro (^values-set! val1 val2 val3)
   `(univ-emit-values-set! ctx ,val1 ,val2 ,val3))
 
+(define-macro (^vect-box type val)
+  `(univ-emit-vect-box ctx ,type ,val))
+
+(define-macro (^vect-unbox type vect)
+  `(univ-emit-vect-unbox ctx ,type ,vect))
+
+(define-macro (^vect? type val)
+  `(univ-emit-vect? ctx ,type ,val))
+
 (define-macro (^vector-box val)
   `(univ-emit-vector-box ctx ,val))
 
@@ -979,6 +1072,15 @@
 (define-macro (^string-set! val1 val2 val3)
   `(univ-emit-string-set! ctx ,val1 ,val2 ,val3))
 
+(define-macro (^substring val1 val2 val3)
+  `(univ-emit-substring ctx ,val1 ,val2 ,val3))
+
+(define-macro (^str-toint val)
+  `(univ-emit-str-toint ctx ,val))
+
+(define-macro (^str-tofloat val)
+  `(univ-emit-str-toint ctx ,val))
+
 (define-macro (^symbol-obj obj force-var?)
   `(univ-emit-symbol-obj ctx ,obj ,force-var?))
 
@@ -1051,8 +1153,8 @@
 (define-macro (^closure-set! val1 val2 val3)
   `(univ-emit-closure-set! ctx ,val1 ,val2 ,val3))
 
-(define-macro (^new-promise expr)
-  `(univ-emit-new-promise ctx ,expr))
+(define-macro (^new-delay-promise expr)
+  `(univ-emit-new-delay-promise ctx ,expr))
 
 (define-macro (^promise? val)
   `(univ-emit-promise? ctx ,val))
@@ -1063,5 +1165,14 @@
 (define-macro (^will? val)
   `(univ-emit-will? ctx ,val))
 
+(define-macro (^new-foreign expr1 expr2)
+  `(univ-emit-new-foreign ctx ,expr1 ,expr2))
+
+(define-macro (^foreign? val)
+  `(univ-emit-foreign? ctx ,val))
+
 (define-macro (^popcount! arg)
   `(univ-emit-popcount! ctx ,arg))
+
+(define-macro (^host-primitive? arg)
+  `(univ-emit-host-primitive? ctx ,arg))
