@@ -2,7 +2,7 @@
 
 (define (test expected actual)
 ;;  (println actual)
-  (println (##fl= expected actual)))
+  (println (##fleqv? expected actual)))
 
 (test   3.90625    (##fl*  3.125  1.25))
 (test  -3.90625    (##fl*  3.125 -1.25))
@@ -14,3 +14,8 @@
 (test   3.90625    (##fl* 3.125 1.25))
 (test  10.7421875  (##fl* 3.125 1.25 2.75))
 (test -37.59765625 (##fl* 3.125 1.25 2.75 -3.5))
+
+(test     0. (##fl*  0.0  0.0))
+(test    -0. (##fl*  0.0 -0.0))
+(test    -0. (##fl* -0.0  0.0))
+(test     0. (##fl* -0.0 -0.0))
