@@ -2,7 +2,7 @@
 
 (define (test expected actual)
 ;;  (println actual)
-  (println (##fl= expected actual)))
+  (println (##fleqv? expected actual)))
 
 (test  1.875 (##fl-  3.125  1.25))
 (test  4.375 (##fl-  3.125 -1.25))
@@ -13,3 +13,8 @@
 (test  1.875 (##fl- 3.125 1.25))
 (test -1.875 (##fl- 3.125 1.25 3.75))
 (test  1.625 (##fl- 3.125 1.25 3.75 -3.5))
+
+(test     0. (##fl-  0.0  0.0))
+(test     0. (##fl-  0.0 -0.0))
+(test    -0. (##fl- -0.0  0.0))
+(test     0. (##fl- -0.0 -0.0))
