@@ -2,7 +2,7 @@
 
 ;;; File: "main.scm"
 
-;;; Copyright (c) 1994-2016 by Marc Feeley, All Rights Reserved.
+;;; Copyright (c) 1994-2018 by Marc Feeley, All Rights Reserved.
 
 ;;;----------------------------------------------------------------------------
 
@@ -23,6 +23,7 @@
                (lambda (script-line script-path) #f)
                #f
                #f
+               #f
                #f)))
 
     (or (try (macro-initialization-file))
@@ -100,6 +101,7 @@
                       script-callback
                       #t
                       #t
+                      #f
                       #f)
 
               (if starter
@@ -112,9 +114,9 @@
 
   (define (compiler-batch-mode options arguments target)
 
-    (define (c-file? file)
+    (define (target-file? file)
       (##assoc (##path-extension file)
-               (c#target-file-extensions (c#target-get 'C))))
+               (c#target-file-extensions target)))
 
     (define (obj-file? file)
       (##string=? (##path-extension file) ##os-obj-extension-string-saved))
@@ -167,7 +169,7 @@
                           ((obj-file? file)
                            (loop1 rest
                                   nb-output-files))
-                          ((c-file? file)
+                          ((target-file? file)
                            (loop1 rest
                                   (if (and (##eq? type 'obj)
                                            (c#get-link-info file #f))
@@ -196,6 +198,16 @@
                           (##assq 'prelude options))
                          (post
                           (##assq 'postlude options))
+                         (module-name
+                          (let ((x (##assq 'module-name options)))
+                            (if x
+                                (##cadr x)
+                                #f)))
+                         (linker-name
+                          (let ((x (##assq 'linker-name options)))
+                            (if x
+                                (##cadr x)
+                                #f)))
                          (cc-options
                           (let ((x (##assq 'cc-options options)))
                             (if x
@@ -260,6 +272,10 @@
                               (##cons obj-file
                                       rev-obj-files)))
 
+                      (define (add-obj-file-at-head obj-file)
+                        (set! rev-obj-files
+                              (##append rev-obj-files (##list obj-file))))
+
                       (define (add-tmp-file tmp-file)
                         (set! rev-tmp-files
                               (##cons tmp-file
@@ -272,7 +288,7 @@
 
                       (define (exit-abnormally)
                         (cleanup)
-                        (##exit-abnormally))
+                        (##exit-abruptly))
 
                       (define (handling file)
                         (if (##fx< 1 nb-output-files)
@@ -289,12 +305,16 @@
                                  file
                                  options: opts
                                  output: output
+                                 module-name: module-name
+                                 linker-name: linker-name
                                  cc-options: cc-options
                                  ld-options-prelude: ld-options-prelude
                                  ld-options: ld-options)
                                 (compile-file
                                  file
                                  options: opts
+                                 module-name: module-name
+                                 linker-name: linker-name
                                  cc-options: cc-options
                                  ld-options-prelude: ld-options-prelude
                                  ld-options: ld-options))
@@ -306,10 +326,14 @@
                                 (compile-file-to-target
                                  file
                                  options: opts
-                                 output: output)
+                                 output: output
+                                 module-name: module-name
+                                 linker-name: linker-name)
                                 (compile-file-to-target
                                  file
-                                 options: opts))
+                                 options: opts
+                                 module-name: module-name
+                                 linker-name: linker-name))
                             (exit-abnormally)))
 
                       (define (do-build-executable obj-files output-filename)
@@ -349,7 +373,7 @@
                                           (else
                                            (loop2 rest))))
                                   (let ((root (##path-strip-extension file)))
-                                    (cond ((c-file? file)
+                                    (cond ((target-file? file)
                                            (if (##memq type '(exe obj))
                                                (let ((obj-file
                                                       (do-compile-file
@@ -374,15 +398,10 @@
                                           (else
                                            (case type
                                              ((dyn)
-                                              (if (##eq? (c#target-name target) 'C)
-                                                  (do-compile-file
-                                                   file
-                                                   options
-                                                   output)
-                                                  (do-compile-file-to-target
-                                                   file
-                                                   options
-                                                   output)))
+                                              (do-compile-file
+                                               file
+                                               options
+                                               output))
                                              ((obj)
                                               (let ((obj-file
                                                      (do-compile-file
@@ -398,7 +417,7 @@
                                                       (and (##eq? type 'c)
                                                            output))))
                                                 (add-gen-file gen-file)
-                                                (if (and (c-file? gen-file)
+                                                (if (and (target-file? gen-file)
                                                          (##eq? type 'exe))
                                                     (let ((obj-file
                                                            (do-compile-file
@@ -440,8 +459,10 @@
                                                              link?)
                                                         (link-flat gen-files
                                                                    output: output
+                                                                   linker-name: linker-name
                                                                    warnings?: warnings-opt?)
                                                         (link-flat gen-files
+                                                                   linker-name: linker-name
                                                                    warnings?: warnings-opt?))
                                                     (if (and output
                                                              link?)
@@ -449,19 +470,23 @@
                                                             (link-incremental
                                                              gen-files
                                                              output: output
+                                                             linker-name: linker-name
                                                              base: base
                                                              warnings?: warnings-opt?)
                                                             (link-incremental
                                                              gen-files
                                                              output: output
+                                                             linker-name: linker-name
                                                              warnings?: warnings-opt?))
                                                         (if base
                                                             (link-incremental
                                                              gen-files
+                                                             linker-name: linker-name
                                                              base: base
                                                              warnings?: warnings-opt?)
                                                             (link-incremental
                                                              gen-files
+                                                             linker-name: linker-name
                                                              warnings?: warnings-opt?))))))
                                           (and link-file
                                                (begin
@@ -472,15 +497,17 @@
                                                              link-file
                                                              (##cons '(obj) options)
                                                              #f)))
-                                                       (add-obj-file obj-link-file)
+                                                       (add-obj-file-at-head obj-link-file)
                                                        (add-tmp-file obj-link-file)
                                                        (if (##not (##assq 'keep-c options))
                                                            (add-tmp-file link-file))))))))
 
+
                                     (if exe?
                                         (and (##pair? rev-obj-files)
                                              (let ((obj-files
                                                     (##reverse rev-obj-files)))
+
                                                (do-build-executable
                                                 obj-files
                                                 (let ((expanded-output
@@ -499,7 +526,12 @@
                                                               (##reverse gen-files)))
                                                             (##car
                                                              rev-obj-files)))
-                                                       ##os-exe-extension-string-saved))))))))
+                                                       (if (##eq?
+                                                            (c#target-name
+                                                             target)
+                                                            'C)
+                                                           ##os-exe-extension-string-saved
+                                                           ##os-bat-extension-string-saved)))))))))
 
                                   (if flat?
                                       (warn-flat-and-not-link-or-exe)))
@@ -511,27 +543,37 @@
     (##repl
      (lambda (first output-port)
        (##write-string
-        "*** WARNING -- Missing argument for option "
+        "*** WARNING -- Missing argument for -"
         output-port)
-       (##write (##symbol->string opt-sym) output-port)
-       (##write-string "\n" output-port)
+       (##write-string (##symbol->string opt-sym) output-port)
+       (##write-string " option\n" output-port)
+       #t)))
+
+  (define (warn-fixnum-argument-expected-for-option opt-sym)
+    (##repl
+     (lambda (first output-port)
+       (##write-string
+        "*** WARNING -- Fixnum argument expected for -"
+        output-port)
+       (##write-string (##symbol->string opt-sym) output-port)
+       (##write-string " option\n" output-port)
        #t)))
 
   (define (warn-unknown-option opt-sym)
     (##repl
      (lambda (first output-port)
        (##write-string
-        "*** WARNING -- Unknown or improperly placed option: "
+        "*** WARNING -- Unknown or improperly placed -"
         output-port)
-       (##write (##symbol->string opt-sym) output-port)
-       (##newline output-port)
+       (##write-string (##symbol->string opt-sym) output-port)
+       (##write-string " option\n" output-port)
        #t)))
 
   (define (warn-multiple-output-files-and-o-option)
     (##repl
      (lambda (first output-port)
        (##write-string
-        "*** WARNING -- Multiple output files: non-directory \"o\" option ignored\n"
+        "*** WARNING -- Multiple output files: non-directory -o option ignored\n"
         output-port)
        #t)))
 
@@ -539,7 +581,7 @@
     (##repl
      (lambda (first output-port)
        (##write-string
-        "*** WARNING -- No incremental link: \"l\" option ignored\n"
+        "*** WARNING -- No incremental link: -l option ignored\n"
         output-port)
        #t)))
 
@@ -547,7 +589,7 @@
     (##repl
      (lambda (first output-port)
        (##write-string
-        "*** WARNING -- \"link\" or \"exe\" options were not specified: \"flat\" option ignored\n"
+        "*** WARNING -- -link or -exe options were not specified: -flat option ignored\n"
         output-port)
        #t)))
 
@@ -555,7 +597,7 @@
     (##repl
      (lambda (first output-port)
        (##write-string
-        "*** WARNING -- The options \"c\", \"link\", \"dynamic\", \"exe\" and \"obj\" are mutually exclusive\n"
+        "*** WARNING -- The -c, -link, -dynamic, -exe and -obj options are mutually exclusive\n"
         output-port)
        #t)))
 
@@ -563,7 +605,7 @@
     (##repl
      (lambda (first output-port)
        (##write-string
-        "*** WARNING -- The options \"preload\" and \"nopreload\" must be used with the options \"link\" and \"exe\"\n"
+        "*** WARNING -- The -preload and -nopreload options must be used with the -link and -exe options\n"
         output-port)
        #t)))
 
@@ -601,10 +643,21 @@
                      (let ((opt-val (##car rest)))
                        (case (##cadr x)
                          ((symbol)
-                          (loop1 (##cdr rest)
-                                 (##cons (##list opt-sym
-                                                 (##string->symbol opt-val))
-                                         rev-options)))
+                          (let ((val (##string->symbol opt-val)))
+                            (loop1 (##cdr rest)
+                                   (##cons (##list opt-sym val)
+                                           rev-options))))
+                         ((fixnum)
+                          (let ((val (##string->number opt-val)))
+                            (if (##fixnum? val)
+                                (loop1 (##cdr rest)
+                                       (##cons (##list opt-sym val)
+                                               rev-options))
+                                (begin
+                                  (if warn?
+                                      (warn-fixnum-argument-expected-for-option opt-sym))
+                                  (loop1 (##cdr rest)
+                                         rev-options)))))
                          (else
                           (loop1 (##cdr rest)
                                  (##cons (##list opt-sym
@@ -673,11 +726,14 @@
                          '((target symbol)
                            (c) (dynamic) (exe) (obj) (link) (flat)
                            (warnings) (verbose) (report)
-                           (expansion) (gvm) (asm)
-                           (check) (force) (keep-c)
+                           (expansion) (gvm) (cfg) (dg) (asm) (keep-c)
+;;TODO: enable and document when compiler supports these options
+;;                           (type-checking) (no-type-checking)
+;;                           (auto-forcing) (no-auto-forcing)
                            (debug) (debug-location) (debug-source) (debug-environments)
                            (track-scheme)
                            (o string) (l string)
+                           (module-name string) (linker-name string)
                            (prelude string) (postlude string)
                            (cc-options string)
                            (ld-options-prelude string) (ld-options string))))
