@@ -1,13 +1,13 @@
 /* File: "os_base.c" */
 
-/* Copyright (c) 1994-2015 by Marc Feeley, All Rights Reserved. */
+/* Copyright (c) 1994-2017 by Marc Feeley, All Rights Reserved. */
 
 /*
  * This module implements the most basic operating system services.
  */
 
 #define ___INCLUDED_FROM_OS_BASE
-#define ___VERSION 408008
+#define ___VERSION 409003
 #include "gambit.h"
 
 #include "os_base.h"
@@ -221,15 +221,36 @@ ___SIZE_T bytes;)
 }
 
 
+#ifdef ___DEBUG_ALLOC_MEM
+void ___free_mem
+   ___P((void *ptr,
+         int lineno,
+         char *file),
+        (ptr,
+         lineno,
+         file)
+void *ptr;
+int lineno;
+char *file;)
+#else
 void ___free_mem
    ___P((void *ptr),
         (ptr)
 void *ptr;)
+#endif
 {
 #ifdef ___DEBUG_ALLOC_MEM
 
 #ifdef ___DEBUG_LOG
-  ___printf ("%p FREED\n", ptr);
+
+  if (file != 0)
+    ___printf ("%p FREED AT \"%s\"@%d.1\n",
+               ptr,
+               file,
+               lineno);
+  else
+    ___printf ("%p FREED\n", ptr);
+
 #endif
 
 #ifdef USE_WIN32
@@ -341,6 +362,74 @@ void *ptr;)
 }
 
 
+#ifdef ___DEBUG_ALLOC_MEM
+void *___alloc_mem_heap
+   ___P((___SIZE_T bytes,
+         int lineno,
+         char *file),
+        (bytes,
+         lineno,
+         file)
+___SIZE_T bytes;
+int lineno;
+char *file;)
+#else
+void *___alloc_mem_heap
+   ___P((___SIZE_T bytes),
+        (bytes)
+___SIZE_T bytes;)
+#endif
+{
+#ifdef ___SUPPORT_LOWLEVEL_EXEC
+
+  return ___alloc_mem_code (bytes);
+
+#else
+
+#ifdef ___DEBUG_ALLOC_MEM
+  return ___alloc_mem (bytes, lineno, file);
+#else
+  return ___alloc_mem (bytes);
+#endif
+
+#endif
+}
+
+
+#ifdef ___DEBUG_ALLOC_MEM
+void ___free_mem_heap
+   ___P((void *ptr,
+         int lineno,
+         char *file),
+        (ptr,
+         lineno,
+         file)
+void *ptr;
+int lineno;
+char *file;)
+#else
+void ___free_mem_heap
+   ___P((void *ptr),
+        (ptr)
+void *ptr;)
+#endif
+{
+#ifdef ___SUPPORT_LOWLEVEL_EXEC
+
+  ___free_mem_code (ptr);
+
+#else
+
+#ifdef ___DEBUG_ALLOC_MEM
+  ___free_mem (ptr, lineno, file);
+#else
+  ___free_mem (ptr);
+#endif
+
+#endif
+}
+
+
 /*---------------------------------------------------------------------------*/
 
 /* Program startup */
@@ -400,7 +489,7 @@ ___HIDDEN int lang_char_encoding ___PVOID
                   (dot[4]=='8' && dot[5]=='\0')))
             char_encoding = ___CHAR_ENCODING_UTF_8;
 
-          ___free_mem (cvalue);
+          ___FREE_MEM(cvalue);
         }
     }
 
@@ -547,7 +636,7 @@ ___UCS_2STRING **argv_return;)
                                   ___ALLOC_MEM(total_arg_len
                                                * sizeof (___UCS_2)))) == 0)
                 {
-                  ___free_mem (argv);
+                  ___FREE_MEM(argv);
                   return ___FIX(___HEAP_OVERFLOW_ERR);
                 }
             }
@@ -583,6 +672,7 @@ ___UCS_2STRING **argv_return;)
                     {
                       if ((nb_backslashes & 1) == 0)
                         in_double_quotes ^= 1;
+#define PROCESS_PROGRAM_LIKE_OTHER_ARGS
 #ifndef PROCESS_PROGRAM_LIKE_OTHER_ARGS
                       if (argc == 0)
                         nb_backslashes = (nb_backslashes<<1) + 1;
@@ -634,9 +724,9 @@ ___HIDDEN void free_windows_command_line
 ___UCS_2STRING *argv;)
 {
   if (argv[0] != 0)
-    ___free_mem (argv[0]);
+    ___FREE_MEM(argv[0]);
 
-  ___free_mem (argv);
+  ___FREE_MEM(argv);
 }
 
 
