@@ -1,6 +1,6 @@
 /* File: "os_files.c" */
 
-/* Copyright (c) 1994-2016 by Marc Feeley, All Rights Reserved. */
+/* Copyright (c) 1994-2018 by Marc Feeley, All Rights Reserved. */
 
 /*
  * This module implements the operating system specific routines
@@ -8,7 +8,7 @@
  */
 
 #define ___INCLUDED_FROM_OS_FILES
-#define ___VERSION 408008
+#define ___VERSION 409003
 #include "gambit.h"
 
 #include "os_base.h"
@@ -450,7 +450,7 @@ ___SCMOBJ ___os_path_homedir ___PVOID
           else
             ___release_scmobj (result);
 
-          ___free_mem (cstr1);
+          ___FREE_MEM(cstr1);
         }
       else
         {
@@ -689,6 +689,9 @@ ___HIDDEN ___STRING_TYPE(___CONFIG_GAMBITDIR_MAP_CE_SELECT) config_gambitdir_map
 #ifdef ___GAMBITDIR_LIB
   "lib=" ___GAMBITDIR_LIB,
 #endif
+#ifdef ___GAMBITDIR_USERLIB
+  "userlib=" ___GAMBITDIR_USERLIB,
+#endif
 #ifdef ___GAMBITDIR_SHARE
   "share=" ___GAMBITDIR_SHARE,
 #endif
