@@ -1,6 +1,6 @@
 /* File: "os_io.h" */
 
-/* Copyright (c) 1994-2016 by Marc Feeley, All Rights Reserved. */
+/* Copyright (c) 1994-2018 by Marc Feeley, All Rights Reserved. */
 
 #ifndef ___OS_IO_H
 #define ___OS_IO_H
@@ -17,25 +17,28 @@ typedef struct ___device_group_struct
     struct ___device_struct *list; /* list of devices in this group */
   } ___device_group;
 
-
+/*TODO: remove "DEVICE" from names*/
 #define ___NONE_KIND              0
-#define ___OBJECT_KIND            1
-#define ___CHARACTER_KIND         3
-#define ___BYTE_KIND              7
-#define ___DEVICE_KIND            15
-#define ___FILE_DEVICE_KIND       (___DEVICE_KIND+16)
-#define ___PIPE_DEVICE_KIND       (___DEVICE_KIND+32)
-#define ___PROCESS_DEVICE_KIND    (___PIPE_DEVICE_KIND+65536)
-#define ___TTY_DEVICE_KIND        (___DEVICE_KIND+64)
-#define ___SERIAL_DEVICE_KIND     (___DEVICE_KIND+128)
-#define ___TCP_CLIENT_DEVICE_KIND (___DEVICE_KIND+256)
-#define ___TCP_SERVER_DEVICE_KIND (___OBJECT_KIND+512)
-#define ___DIRECTORY_KIND         (___OBJECT_KIND+1024)
-#define ___EVENT_QUEUE_KIND       (___OBJECT_KIND+2048)
-#define ___TIMER_KIND             (___OBJECT_KIND+4096)
-#define ___VECTOR_KIND            (___OBJECT_KIND+8192)
-#define ___STRING_KIND            (___CHARACTER_KIND+16384)
-#define ___U8VECTOR_KIND          (___BYTE_KIND+32768)
+#define ___WAITABLE_KIND          1
+#define ___OBJECT_KIND            (___WAITABLE_KIND+2)
+#define ___CHARACTER_KIND         (___OBJECT_KIND+4)
+#define ___BYTE_KIND              (___CHARACTER_KIND+8)
+#define ___DEVICE_KIND            (___BYTE_KIND+16)
+#define ___FILE_DEVICE_KIND       (___DEVICE_KIND+32)
+#define ___PIPE_DEVICE_KIND       (___DEVICE_KIND+64)
+#define ___PROCESS_DEVICE_KIND    (___PIPE_DEVICE_KIND+131072)
+#define ___TTY_DEVICE_KIND        (___DEVICE_KIND+128)
+#define ___SERIAL_DEVICE_KIND     (___DEVICE_KIND+256)
+#define ___TCP_CLIENT_DEVICE_KIND (___DEVICE_KIND+512)
+#define ___TCP_SERVER_DEVICE_KIND (___OBJECT_KIND+1024)
+#define ___DIRECTORY_KIND         (___OBJECT_KIND+2048)
+#define ___EVENT_QUEUE_KIND       (___OBJECT_KIND+4096)
+#define ___TIMER_KIND             (___OBJECT_KIND+8192)
+#define ___VECTOR_KIND            (___OBJECT_KIND+16384)
+#define ___STRING_KIND            (___CHARACTER_KIND+32768)
+#define ___U8VECTOR_KIND          (___BYTE_KIND+65536)
+#define ___RAW_DEVICE_KIND        (___WAITABLE_KIND+262144)
+#define ___UDP_DEVICE_KIND        (___OBJECT_KIND+524288)
 
 #define ___OPEN_STATE(x)      ((x)&(1<<12))
 #define ___OPEN_STATE_MASK(x) ((x)&~(1<<12))
@@ -91,37 +94,23 @@ typedef struct ___device_struct
 #define MAX_CONDVARS 8192
 #endif
 
-#ifdef USE_poll
-#ifndef MAX_POLLFDS
-#define MAX_POLLFDS MAX_CONDVARS
-#endif
+#ifdef USE_select_or_poll
 
-typedef ___SIZE_TS ___fdbits;
+typedef ___UWORD ___fdbits;
 
-#define ___FDBITS (8 * sizeof (___fdbits))
-#define ___FD_ELT(fd) ((fd) / ___FDBITS)
-#define ___FD_MASK(fd) ((___fdbits) 1 << ((fd) % ___FDBITS))
+#define ___FDBITS ___WORD_WIDTH
+#define ___FD_ELT(fd) ((fd) >> ___LOG_WORD_WIDTH)
+#define ___FD_MASK(fd) ((___fdbits) 1 << ((fd) % ___WORD_WIDTH))
 
-#define ___FD_ZERO(set)                       \
-  memset ((set), 0, sizeof (___poll_fd_set))
+#define ___FD_ZERO(set, sz)                        \
+  memset ((set), 0, sz/8)
 #define ___FD_SET(fd, set)                    \
-  ((set)->fds[___FD_ELT (fd)] |= ___FD_MASK (fd))
+  ((set)[___FD_ELT (fd)] |= ___FD_MASK (fd))
 #define ___FD_CLR(fd, set)                    \
-  ((set)->fds[___FD_ELT (fd)] &= ~___FD_MASK (fd))
+  ((set)[___FD_ELT (fd)] &= ~___FD_MASK (fd))
 #define ___FD_ISSET(fd, set)                  \
-  ((set)->fds[___FD_ELT (fd)] & ___FD_MASK (fd))
+  ((set)[___FD_ELT (fd)] & ___FD_MASK (fd))
 
-typedef struct ___poll_fd_set {
-  ___fdbits fds[MAX_POLLFDS / ___FDBITS];
-} ___poll_fd_set;
-
-#endif
-
-#ifdef USE_select
-#define ___FD_ZERO  FD_ZERO
-#define ___FD_ISSET FD_ISSET
-#define ___FD_CLR   FD_CLR
-#define ___FD_SET   FD_SET
 #endif
 
 typedef struct ___device_select_state_struct
@@ -138,17 +127,17 @@ typedef struct ___device_select_state_struct
 
 #ifdef USE_select
     int highest_fd_plus_1;
-    fd_set readfds;
-    fd_set writefds;
-    fd_set exceptfds;
+    ___fdbits *readfds;
+    ___fdbits *writefds;
+    ___fdbits *exceptfds;
 #endif
 
 #ifdef USE_poll
-    struct pollfd pollfds[MAX_POLLFDS];
+    struct pollfd pollfds[MAX_CONDVARS];
     int pollfd_count;
     /* active set bitmaps */
-    ___poll_fd_set readfds;
-    ___poll_fd_set writefds;
+    ___fdbits *readfds;
+    ___fdbits *writefds;
 #endif
 
 #endif
@@ -167,6 +156,20 @@ typedef struct ___device_select_state_struct
   } ___device_select_state;
 
 
+#ifdef USE_FDSET_RESIZING
+
+extern void ___fdset_resize_pstate
+   ___P((___processor_state ___ps,
+         int maxfd),
+        ());
+
+extern ___BOOL ___fdset_resize
+   ___P((int fd1,
+         int fd2),
+        ());
+
+#endif
+
 extern void ___device_select_add_relative_timeout
    ___P((___device_select_state *state,
          int i,
@@ -657,7 +660,7 @@ extern ___SCMOBJ ___os_device_kind
         ());
 
 extern ___SCMOBJ ___os_device_force_output
-   ___P((___SCMOBJ dev,
+   ___P((___SCMOBJ dev_condvar,
          ___SCMOBJ level),
         ());
 
@@ -671,27 +674,27 @@ extern ___SCMOBJ ___os_device_close
 /* Stream device operations. */
 
 extern ___SCMOBJ ___os_device_stream_seek
-   ___P((___SCMOBJ dev,
+   ___P((___SCMOBJ dev_condvar,
          ___SCMOBJ pos,
          ___SCMOBJ whence),
         ());
 
 extern ___SCMOBJ ___os_device_stream_read
-   ___P((___SCMOBJ dev,
+   ___P((___SCMOBJ dev_condvar,
          ___SCMOBJ buffer,
          ___SCMOBJ lo,
          ___SCMOBJ hi),
         ());
 
 extern ___SCMOBJ ___os_device_stream_write
-   ___P((___SCMOBJ dev,
+   ___P((___SCMOBJ dev_condvar,
          ___SCMOBJ buffer,
          ___SCMOBJ lo,
          ___SCMOBJ hi),
         ());
 
 extern ___SCMOBJ ___os_device_stream_width
-   ___P((___SCMOBJ dev),
+   ___P((___SCMOBJ dev_condvar),
         ());
 
 extern ___SCMOBJ ___os_device_stream_default_options
@@ -741,15 +744,20 @@ extern ___SCMOBJ ___os_device_process_status
    ___P((___SCMOBJ dev),
         ());
 
+extern void ___cleanup_child_interrupt_handling ___PVOID;
+
 /*   -   -   -   -   -   -   -   -   -   -   -   -   -   -   -   -   -   -   */
 
 /* Opening a TCP client. */
 
 extern ___SCMOBJ ___os_device_tcp_client_open
-   ___P((___SCMOBJ server_addr,
+   ___P((___SCMOBJ local_addr,
+         ___SCMOBJ local_port_num,
+         ___SCMOBJ addr,
          ___SCMOBJ port_num,
          ___SCMOBJ options,
-         ___SCMOBJ tls_context),
+         ___SCMOBJ tls_context,
+         ___SCMOBJ server_name),
         ());
 
 extern ___SCMOBJ ___os_device_tcp_client_socket_info
@@ -762,15 +770,15 @@ extern ___SCMOBJ ___os_device_tcp_client_socket_info
 /* Opening and reading a TCP server. */
 
 extern ___SCMOBJ ___os_device_tcp_server_open
-   ___P((___SCMOBJ server_addr,
-         ___SCMOBJ port_num,
+   ___P((___SCMOBJ local_addr,
+         ___SCMOBJ local_port_num,
          ___SCMOBJ backlog,
          ___SCMOBJ options,
          ___SCMOBJ tls_context),
         ());
 
 extern ___SCMOBJ ___os_device_tcp_server_read
-   ___P((___SCMOBJ dev),
+   ___P((___SCMOBJ dev_condvar),
         ());
 
 extern ___SCMOBJ ___os_device_tcp_server_socket_info
@@ -793,6 +801,41 @@ extern ___SCMOBJ ___os_make_tls_context
 
 /*   -   -   -   -   -   -   -   -   -   -   -   -   -   -   -   -   -   -   */
 
+/* Opening a UDP socket. */
+
+extern ___SCMOBJ ___os_device_udp_open
+   ___P((___SCMOBJ local_addr,
+         ___SCMOBJ local_port_num,
+         ___SCMOBJ options),
+        ());
+
+extern ___SCMOBJ ___os_device_udp_read_subu8vector
+   ___P((___SCMOBJ dev_condvar,
+         ___SCMOBJ buffer,
+         ___SCMOBJ lo,
+         ___SCMOBJ hi),
+        ());
+
+extern ___SCMOBJ ___os_device_udp_write_subu8vector
+   ___P((___SCMOBJ dev_condvar,
+         ___SCMOBJ buffer,
+         ___SCMOBJ lo,
+         ___SCMOBJ hi),
+        ());
+
+extern ___SCMOBJ ___os_device_udp_destination_set
+   ___P((___SCMOBJ dev_condvar,
+         ___SCMOBJ addr,
+         ___SCMOBJ port_num),
+        ());
+
+extern ___SCMOBJ ___os_device_udp_socket_info
+   ___P((___SCMOBJ dev,
+         ___SCMOBJ source),
+        ());
+
+/*   -   -   -   -   -   -   -   -   -   -   -   -   -   -   -   -   -   -   */
+
 /* Opening and reading a directory. */
 
 extern ___SCMOBJ ___os_device_directory_open_path
@@ -801,7 +844,7 @@ extern ___SCMOBJ ___os_device_directory_open_path
         ());
 
 extern ___SCMOBJ ___os_device_directory_read
-   ___P((___SCMOBJ dev),
+   ___P((___SCMOBJ dev_condvar),
         ());
 
 /*   -   -   -   -   -   -   -   -   -   -   -   -   -   -   -   -   -   -   */
@@ -813,15 +856,23 @@ extern ___SCMOBJ ___os_device_event_queue_open
         ());
 
 extern ___SCMOBJ ___os_device_event_queue_read
-   ___P((___SCMOBJ dev),
+   ___P((___SCMOBJ dev_condvar),
         ());
 
+/* Opening a raw device (file descriptor) */
+
+extern ___SCMOBJ ___os_device_raw_open_from_fd
+   ___P((___SCMOBJ fd,
+         ___SCMOBJ flags),
+        ());
+
+
 /*   -   -   -   -   -   -   -   -   -   -   -   -   -   -   -   -   -   -   */
 
 /* Waiting for I/O to become possible on a set of devices. */
 
 extern ___SCMOBJ ___os_condvar_select
-   ___P((___SCMOBJ run_queue,
+   ___P((___SCMOBJ devices,
          ___SCMOBJ timeout),
         ());
 
