@@ -1,6 +1,6 @@
 /* File: "os_time.c" */
 
-/* Copyright (c) 1994-2016 by Marc Feeley, All Rights Reserved. */
+/* Copyright (c) 1994-2018 by Marc Feeley, All Rights Reserved. */
 
 /*
  * This module implements the operating system specific routines
@@ -8,7 +8,7 @@
  */
 
 #define ___INCLUDED_FROM_OS_TIME
-#define ___VERSION 408008
+#define ___VERSION 409003
 #include "gambit.h"
 
 #include "os_thread.h"
@@ -25,7 +25,7 @@ ___time_module ___time_mod =
   TIME_POS_INFINITY,
   TIME_NEG_INFINITY,
   0.0,
-  -1.0,
+  -999.0, /* indicates that heartbeat interval has never been set */
   NULL
 
 #ifdef ___TIME_MODULE_INIT
@@ -919,17 +919,21 @@ ___HIDDEN void cleanup_process_times ___PVOID
 #ifndef USE_CreateThread
 
 
-___F64 ___set_heartbeat_interval
-   ___P((___F64 seconds),
-        (seconds)
-___F64 seconds;)
+___F64 ___get_heartbeat_interval ___PVOID
 {
-  ___time_mod.current_heartbeat_interval = seconds;
   return 0.0;
 }
 
 
-void ___mask_heartbeat_interrupts_begin
+void ___set_heartbeat_interval
+   ___P((___F64 seconds),
+        (seconds)
+___F64 seconds;)
+{
+}
+
+
+___EXP_FUNC(void,___mask_heartbeat_interrupts_begin)
    ___P((___mask_heartbeat_interrupts_state *state),
         (state)
 ___mask_heartbeat_interrupts_state *state;)
@@ -937,7 +941,7 @@ ___mask_heartbeat_interrupts_state *state;)
 }
 
 
-void ___mask_heartbeat_interrupts_end
+___EXP_FUNC(void,___mask_heartbeat_interrupts_end)
    ___P((___mask_heartbeat_interrupts_state *state),
         (state)
 ___mask_heartbeat_interrupts_state *state;)
@@ -979,15 +983,19 @@ int sig;)
 }
 
 
-___F64 ___set_heartbeat_interval
+___F64 ___get_heartbeat_interval ___PVOID
+{
+  return ___time_mod.current_heartbeat_interval;
+}
+
+
+void ___set_heartbeat_interval
    ___P((___F64 seconds),
         (seconds)
 ___F64 seconds;)
 {
   struct itimerval tv;
 
-  ___time_mod.current_heartbeat_interval = seconds;
-
   if (seconds < 0.0) /* turn heartbeat off */
     {
       tv.it_interval.tv_sec  = 0;
@@ -995,7 +1003,7 @@ ___F64 seconds;)
       tv.it_value.tv_sec     = 0;
       tv.it_value.tv_usec    = 0;
       setitimer (HEARTBEAT_ITIMER, &tv, 0);
-      return 0.0;
+      ___time_mod.current_heartbeat_interval = 0.0;
     }
   else
     {
@@ -1024,37 +1032,33 @@ ___F64 seconds;)
       tv.it_value.tv_usec    = usecs;
       setitimer (HEARTBEAT_ITIMER, &tv, 0);
       getitimer (HEARTBEAT_ITIMER, &tv);
-      return tv.it_interval.tv_sec + tv.it_interval.tv_usec / 1000000.0;
+      ___time_mod.current_heartbeat_interval =
+        tv.it_interval.tv_sec + tv.it_interval.tv_usec / 1000000.0;
     }
 }
 
 
-void ___mask_heartbeat_interrupts_begin
+___EXP_FUNC(void,___mask_heartbeat_interrupts_begin)
    ___P((___mask_heartbeat_interrupts_state *state),
         (state)
 ___mask_heartbeat_interrupts_state *state;)
 {
 #ifdef USE_POSIX
 
-  ___sigset_type toblock;
-
-  sigemptyset (&toblock);
-  sigaddset (&toblock, HEARTBEAT_SIG);
-
-  ___thread_sigmask (SIG_BLOCK, &toblock, &state->oldmask);
+  ___thread_sigmask1 (SIG_BLOCK, HEARTBEAT_SIG, state->sigset+0);
 
 #endif
 }
 
 
-void ___mask_heartbeat_interrupts_end
+___EXP_FUNC(void,___mask_heartbeat_interrupts_end)
    ___P((___mask_heartbeat_interrupts_state *state),
         (state)
 ___mask_heartbeat_interrupts_state *state;)
 {
 #ifdef USE_POSIX
 
-  ___thread_sigmask (SIG_SETMASK, &state->oldmask, 0);
+  ___thread_sigmask (SIG_SETMASK, state->sigset+0, NULL);
 
 #endif
 }
@@ -1062,10 +1066,12 @@ ___mask_heartbeat_interrupts_state *state;)
 
 ___SCMOBJ ___setup_heartbeat_interrupt_handling ___PVOID
 {
-  ___set_heartbeat_interval (___time_mod.current_heartbeat_interval);
-
 #ifdef USE_POSIX
+
   ___set_signal_handler (HEARTBEAT_SIG, heartbeat_interrupt_handler);
+
+  ___thread_sigmask1 (SIG_UNBLOCK, HEARTBEAT_SIG, NULL);
+
 #endif
 
   return ___FIX(___NO_ERR);
@@ -1074,14 +1080,14 @@ ___SCMOBJ ___setup_heartbeat_interrupt_handling ___PVOID
 
 void ___cleanup_heartbeat_interrupt_handling ___PVOID
 {
-  ___F64 save_heartbeat_interval = ___time_mod.current_heartbeat_interval;
-
   ___set_heartbeat_interval (-1.0);
 
-  ___time_mod.current_heartbeat_interval = save_heartbeat_interval;
-
 #ifdef USE_POSIX
+
   ___set_signal_handler (HEARTBEAT_SIG, SIG_IGN);
+
+  ___thread_sigmask1 (SIG_UNBLOCK, HEARTBEAT_SIG, NULL);
+
 #endif
 
 }
@@ -1110,7 +1116,13 @@ ___HIDDEN void __interrupt __far heartbeat_interrupt_handler ___PVOID
 }
 
 
-___F64 ___set_heartbeat_interval
+___F64 ___get_heartbeat_interval ___PVOID
+{
+  return ___time_mod.current_heartbeat_interval;
+}
+
+
+void ___set_heartbeat_interval
    ___P((___F64 seconds),
         (seconds)
 ___F64 seconds;)
@@ -1120,7 +1132,7 @@ ___F64 seconds;)
   if (seconds < 0.0) /* turn heartbeat off */
     {
       _dos_setvect (0x1C, prev_vector_1Ch);
-      return 0.0;
+      ___time_mod.current_heartbeat_interval = 0.0;
     }
   else
     {
@@ -1139,26 +1151,27 @@ ___F64 seconds;)
       ___time_mod.heartbeat_interval = t;
       ___time_mod.heartbeat_countdown = ___time_mod.heartbeat_interval;
       _dos_setvect (0x1C, ___time_mod.heartbeat_interrupt_handler);
-      return t / 18.2;
+      ___time_mod.current_heartbeat_interval = t / 18.2;
     }
 }
 
 
-void ___mask_heartbeat_interrupts_begin
+___EXP_FUNC(void,___mask_heartbeat_interrupts_begin)
    ___P((___mask_heartbeat_interrupts_state *state),
         (state)
 ___mask_heartbeat_interrupts_state *state;)
 {
+  *___CAST(int*,state) = ___time_mod.heartbeat_enabled;
   ___time_mod.heartbeat_enabled = 0;
 }
 
 
-void ___mask_heartbeat_interrupts_end
+___EXP_FUNC(void,___mask_heartbeat_interrupts_end)
    ___P((___mask_heartbeat_interrupts_state *state),
         (state)
 ___mask_heartbeat_interrupts_state *state;)
 {
-  ___time_mod.heartbeat_enabled = 1;
+  ___time_mod.heartbeat_enabled = *___CAST(int*,state);
 }
 
 
@@ -1204,7 +1217,13 @@ ULONG param;)
 }
 
 
-___F64 ___set_heartbeat_interval
+___F64 ___get_heartbeat_interval ___PVOID
+{
+  return ___time_mod.current_heartbeat_interval;
+}
+
+
+void ___set_heartbeat_interval
    ___P((___F64 seconds),
         (seconds)
 ___F64 seconds;)
@@ -1212,14 +1231,17 @@ ___F64 seconds;)
   ___time_mod.current_heartbeat_interval = seconds;
 
   if (!___time_mod.heartbeat_hev || !___time_mod.heartbeat_tid)
-    return 0.0;
+    {
+      ___time_mod.current_heartbeat_interval = 0.0;
+      return;
+    }
   if (___time_mod.heartbeat_htimer)
     {
       DosStopTimer (___time_mod.htimer);
       ___time_mod.heartbeat_htimer = 0;
     }
   if (seconds < 0.0) /* turn heartbeat off */
-    return 0.0;
+    ___time_mod.current_heartbeat_interval = 0.0;
   else
     {
       ___F64 msecs = seconds * 1000.0;
@@ -1238,29 +1260,30 @@ ___F64 seconds;)
           == NO_ERROR)
         {
           ___time_mod.heartbeat_htimer = 1;
-          return m / 1000.0;
+          ___time_mod.current_heartbeat_interval = m / 1000.0;
         }
       else
-        return 0.0;
+        ___time_mod.current_heartbeat_interval = 0.0;
     }
 }
 
 
-void ___mask_heartbeat_interrupts_begin
+___EXP_FUNC(void,___mask_heartbeat_interrupts_begin)
    ___P((___mask_heartbeat_interrupts_state *state),
         (state)
 ___mask_heartbeat_interrupts_state *state;)
 {
+  *___CAST(int*,state) = ___time_mod.heartbeat_enabled;
   ___time_mod.heartbeat_enabled = 0;
 }
 
 
-void ___mask_heartbeat_interrupts_end
+___EXP_FUNC(void,___mask_heartbeat_interrupts_end)
    ___P((___mask_heartbeat_interrupts_state *state),
         (state)
 ___mask_heartbeat_interrupts_state *state;)
 {
-  ___time_mod.heartbeat_enabled = 1;
+  ___time_mod.heartbeat_enabled = *___CAST(int*,state);
 }
 
 
@@ -1315,12 +1338,6 @@ void ___cleanup_heartbeat_interrupt_handling ___PVOID
 #ifdef USE_VInstall
 
 
-___HIDDEN ___BOOL heartbeat_task_installed;
-___HIDDEN short heartbeat_task_ticks;
-___HIDDEN VBLTask heartbeat_task;
-___HIDDEN ___BOOL heartbeat_enabled;
-
-
 ___HIDDEN void heartbeat_task_code ___PVOID
 {
   if (___time_mod.heartbeat_enabled)
@@ -1347,7 +1364,13 @@ ___HIDDEN asm void heartbeat_task_proc ___PVOID
 #endif
 
 
-___F64 ___set_heartbeat_interval
+___F64 ___get_heartbeat_interval ___PVOID
+{
+  return ___time_mod.current_heartbeat_interval;
+}
+
+
+void ___set_heartbeat_interval
    ___P((___F64 seconds),
         (seconds)
 ___F64 seconds;)
@@ -1358,11 +1381,12 @@ ___F64 seconds;)
     if (VRemove (___CAST(QElemPtr,&___time_mod.heartbeat_task)) != noErr)
       {
         ___time_mod.heartbeat_task_installed = 0;
-        return 0.0;
+        ___time_mod.current_heartbeat_interval = 0.0;
+        return;
       }
 
   if (seconds < 0.0) /* turn heartbeat off */
-    return 0.0;
+    ___time_mod.current_heartbeat_interval = 0.0;
   else
     {
       ___F64 ticks = seconds * 60.0; /* 60 ticks/sec */
@@ -1385,29 +1409,30 @@ ___F64 seconds;)
       if (VInstall (___CAST(QElemPtr,&___time_mod.heartbeat_task)) == noErr)
         {
           ___time_mod.heartbeat_task_installed = 1;
-          return t / 60.0;
+          ___time_mod.current_heartbeat_interval = t / 60.0;
         }
       else
-        return 0.0;
+        ___time_mod.current_heartbeat_interval = 0.0;
     }
 }
 
 
-void ___mask_heartbeat_interrupts_begin
+___EXP_FUNC(void,___mask_heartbeat_interrupts_begin)
    ___P((___mask_heartbeat_interrupts_state *state),
         (state)
 ___mask_heartbeat_interrupts_state *state;)
 {
+  *___CAST(int*,state) = ___time_mod.heartbeat_enabled;
   ___time_mod.heartbeat_enabled = 0;
 }
 
 
-void ___mask_heartbeat_interrupts_end
+___EXP_FUNC(void,___mask_heartbeat_interrupts_end)
    ___P((___mask_heartbeat_interrupts_state *state),
         (state)
 ___mask_heartbeat_interrupts_state *state;)
 {
-  ___time_mod.heartbeat_enabled = 1;
+  ___time_mod.heartbeat_enabled = *___CAST(int*,state);
 }
 
 
@@ -1476,7 +1501,13 @@ LPVOID param;)
 }
 
 
-___F64 ___set_heartbeat_interval
+___F64 ___get_heartbeat_interval ___PVOID
+{
+  return ___time_mod.current_heartbeat_interval;
+}
+
+
+void ___set_heartbeat_interval
    ___P((___F64 seconds),
         (seconds)
 ___F64 seconds;)
@@ -1489,13 +1520,11 @@ ___F64 seconds;)
    * function above.
    */
 
-  ___time_mod.current_heartbeat_interval = seconds;
-
   if (seconds < 0.0) /* turn heartbeat off */
     {
       ___time_mod.heartbeat_interval = INFINITE;
       SetEvent (___time_mod.heartbeat_update); /* ignore error */
-      return 0.0;
+      ___time_mod.current_heartbeat_interval = 0.0;
     }
   else
     {
@@ -1513,26 +1542,27 @@ ___F64 seconds;)
 
       ___time_mod.heartbeat_interval = m;
       SetEvent (___time_mod.heartbeat_update); /* ignore error */
-      return m / 1000.0;
+      ___time_mod.current_heartbeat_interval = m / 1000.0;
     }
 }
 
 
-void ___mask_heartbeat_interrupts_begin
+___EXP_FUNC(void,___mask_heartbeat_interrupts_begin)
    ___P((___mask_heartbeat_interrupts_state *state),
         (state)
 ___mask_heartbeat_interrupts_state *state;)
 {
+  *___CAST(int*,state) = ___time_mod.heartbeat_enabled;
   ___time_mod.heartbeat_enabled = 0;
 }
 
 
-void ___mask_heartbeat_interrupts_end
+___EXP_FUNC(void,___mask_heartbeat_interrupts_end)
    ___P((___mask_heartbeat_interrupts_state *state),
         (state)
 ___mask_heartbeat_interrupts_state *state;)
 {
-  ___time_mod.heartbeat_enabled = 1;
+  ___time_mod.heartbeat_enabled = *___CAST(int*,state);
 }
 
 
