@@ -1,6 +1,6 @@
 ; File: "error.scm"
 
-; Copyright (c) 1998-2015 by Marc Feeley, All Rights Reserved.
+; Copyright (c) 1998-2018 by Marc Feeley, All Rights Reserved.
 
 ; Test program for error processing.
 
@@ -11,7 +11,7 @@
 (define scheme-system
   (let ((str1 (symbol->string (car '(aB\c;
                                      ))))
-        (str2 "\0411\x23"))
+        (str2 "\0411\x23;"))
     (cond ((or (equal? str1 "aB\\c") (equal? str1 "aB"))
            'gambit) ; Gambit in case-sensitive mode
           ((equal? str1 "ab\\c")
@@ -151,10 +151,10 @@
                  (call "???" '()))
                 ((heap-overflow-exception? exc)
                  (call "???" '()))
-                ((improper-length-list-exception? exc)
+                ((length-mismatch-exception? exc)
                  (call
-                  (improper-length-list-exception-procedure exc)
-                  (improper-length-list-exception-arguments exc)))
+                  (length-mismatch-exception-procedure exc)
+                  (length-mismatch-exception-arguments exc)))
                 ((join-timeout-exception? exc)
                  (call
                   (join-timeout-exception-procedure exc)
@@ -1665,7 +1665,7 @@
 
 (define (test-make-string)
 (try 'make-string make-string 0)
-(try 'make-string make-string 3)
+;;(try 'make-string make-string 3)
 (try 'make-string make-string 536870911)
 (try 'make-string make-string 12345678901234567890)
 (try 'make-string make-string -1)
