@@ -101,6 +101,12 @@
 (check-eqv? (##s32vector-ref v5 0) 2147483647)
 (check-eqv? (##s32vector-ref v5 1) 2147483647)
 
+(check-equal? (##s32vector-set v2 1 99) '#s32(-2147483648 99 0 1 2147483647))
+(check-equal? v2 '#s32(-2147483648 -2 0 1 2147483647))
+(check-equal? (##s32vector-set v4 1 99) '#s32(-2147483648 99))
+(check-equal? (##s32vector-set v5 1 99) '#s32(2147483647 99))
+(check-equal? (##s32vector-set '#s32(11 22 33) 0 99) '#s32(99 22 33))
+
 (check-eq? (##s32vector-set! v2 1 99) v2)
 (check-eq? (##s32vector-set! v3 1 99) v3)
 (check-eq? (##s32vector-set! v4 1 99) v4)
@@ -198,6 +204,10 @@
 
 (check-equal? (s32vector->list '#s32()) '())
 (check-equal? (s32vector->list v6) '(-2147483648 -2 0 1 2147483647))
+(check-equal? (s32vector->list v6 0) '(-2147483648 -2 0 1 2147483647))
+(check-equal? (s32vector->list v6 2) '(0 1 2147483647))
+(check-equal? (s32vector->list v6 2 4) '(0 1))
+(check-equal? (s32vector->list v6 0 5) '(-2147483648 -2 0 1 2147483647))
 (check-equal? (s32vector->list v7) '(0 0))
 
 (check-equal? (list->s32vector '()) '#s32())
@@ -213,6 +223,14 @@
 
 (check-equal? (s32vector-copy '#s32()) '#s32())
 (check-equal? (s32vector-copy v6) v6)
+(check-equal? (s32vector-copy v6 0) v6)
+(check-equal? (s32vector-copy v6 2) '#s32(0 1 2147483647))
+(check-equal? (s32vector-copy v6 0 0) '#s32())
+(check-equal? (s32vector-copy v6 4 4) '#s32())
+(check-equal? (s32vector-copy v6 0 2) '#s32(-2147483648 -2))
+(check-equal? (s32vector-copy v6 2 4) '#s32(0 1))
+(check-equal? (s32vector-copy v6 4 5) '#s32(2147483647))
+(check-equal? (s32vector-copy v6 0 5) v6)
 
 (check-equal? (subs32vector v6 0 0) '#s32())
 (check-equal? (subs32vector v6 4 4) '#s32())
@@ -239,6 +257,12 @@
 (check-eqv? (s32vector-ref v9 0) 2147483647)
 (check-eqv? (s32vector-ref v9 1) 2147483647)
 
+(check-equal? (s32vector-set v6 1 99) '#s32(-2147483648 99 0 1 2147483647))
+(check-equal? v6 '#s32(-2147483648 -2 0 1 2147483647))
+(check-equal? (s32vector-set v8 1 99) '#s32(-2147483648 99))
+(check-equal? (s32vector-set v9 1 99) '#s32(2147483647 99))
+(check-equal? (s32vector-set '#s32(11 22 33) 0 99) '#s32(99 22 33))
+
 (check-eq? (s32vector-set! v6 1 99) (void))
 (check-eq? (s32vector-set! v7 1 99) (void))
 (check-eq? (s32vector-set! v8 1 99) (void))
@@ -275,6 +299,12 @@
 (check-eq? (s32vector-fill! v6 2147483647) (void))
 (check-equal? v6 '#s32(2147483647 2147483647 2147483647))
 
+(check-eq? (s32vector-fill! v6 3 1) (void))
+(check-equal? v6 '#s32(2147483647 3 3))
+
+(check-eq? (s32vector-fill! v6 99 0 2) (void))
+(check-equal? v6 '#s32(99 99 3))
+
 (check-eq? (subs32vector-fill! v6 0 3 9) (void))
 (check-equal? v6 '#s32(9 9 9))
 
@@ -290,15 +320,24 @@
 (check-eq? (subs32vector-move! v9 0 2 v6 1) (void))
 (check-equal? v6 '#s32(2147483647 2147483647 99))
 
-(check-tail-exn type-exception? (lambda () (s32vector 11 bool 22)))
-(check-tail-exn type-exception? (lambda () (s32vector 11 -2147483649 22)))
-(check-tail-exn type-exception? (lambda () (s32vector 11 2147483648 22)))
+(check-eq? (s32vector-copy! v6 0 '#s32(11 22 33)) (void))
+(check-equal? v6 '#s32(11 22 33))
+
+(check-eq? (s32vector-copy! v6 2 '#s32(33 44) 1) (void))
+(check-equal? v6 '#s32(11 22 44))
+
+(check-eq? (s32vector-copy! v6 1 '#s32(55 66 77 88) 0 2) (void))
+(check-equal? v6 '#s32(11 55 66))
+
+(check-tail-exn type-exception? (lambda () (s32vector 11 bool 22))) ;; homovect only
+(check-tail-exn type-exception? (lambda () (s32vector 11 -2147483649 22))) ;; homovect only
+(check-tail-exn type-exception? (lambda () (s32vector 11 2147483648 22))) ;; homovect only
 
 (check-tail-exn type-exception? (lambda () (make-s32vector bool)))
 (check-tail-exn type-exception? (lambda () (make-s32vector bool 11)))
-(check-tail-exn type-exception? (lambda () (make-s32vector 11 bool)))
-(check-tail-exn type-exception? (lambda () (make-s32vector 11 -2147483649)))
-(check-tail-exn type-exception? (lambda () (make-s32vector 11 2147483648)))
+(check-tail-exn type-exception? (lambda () (make-s32vector 11 bool))) ;; homovect only
+(check-tail-exn type-exception? (lambda () (make-s32vector 11 -2147483649))) ;; homovect only
+(check-tail-exn type-exception? (lambda () (make-s32vector 11 2147483648))) ;; homovect only
 (check-tail-exn range-exception? (lambda () (make-s32vector -1 0)))
 
 (check-tail-exn type-exception? (lambda () (s32vector-length bool)))
@@ -315,6 +354,8 @@
 (check-tail-exn type-exception? (lambda () (append-s32vectors '(1 2 3))))
 
 (check-tail-exn type-exception? (lambda () (s32vector-copy bool)))
+(check-tail-exn type-exception? (lambda () (s32vector-copy v9 bool)))
+(check-tail-exn type-exception? (lambda () (s32vector-copy v9 0 bool)))
 
 (check-tail-exn type-exception? (lambda () (subs32vector bool 0 0)))
 (check-tail-exn type-exception? (lambda () (subs32vector v9 bool 0)))
@@ -329,11 +370,19 @@
 (check-tail-exn range-exception? (lambda () (s32vector-ref v5 -1)))
 (check-tail-exn range-exception? (lambda () (s32vector-ref v5 2)))
 
+(check-tail-exn type-exception? (lambda () (s32vector-set bool 0 11)))
+(check-tail-exn type-exception? (lambda () (s32vector-set v5 bool 11)))
+(check-tail-exn type-exception? (lambda () (s32vector-set v5 0 bool))) ;; homovect only
+(check-tail-exn type-exception? (lambda () (s32vector-set v5 0 -2147483649))) ;; homovect only
+(check-tail-exn type-exception? (lambda () (s32vector-set v5 0 2147483648))) ;; homovect only
+(check-tail-exn range-exception? (lambda () (s32vector-set v5 -1 0)))
+(check-tail-exn range-exception? (lambda () (s32vector-set v5 2 0)))
+
 (check-tail-exn type-exception? (lambda () (s32vector-set! bool 0 11)))
 (check-tail-exn type-exception? (lambda () (s32vector-set! v5 bool 11)))
-(check-tail-exn type-exception? (lambda () (s32vector-set! v5 0 bool)))
-(check-tail-exn type-exception? (lambda () (s32vector-set! v5 0 -2147483649)))
-(check-tail-exn type-exception? (lambda () (s32vector-set! v5 0 2147483648)))
+(check-tail-exn type-exception? (lambda () (s32vector-set! v5 0 bool))) ;; homovect only
+(check-tail-exn type-exception? (lambda () (s32vector-set! v5 0 -2147483649))) ;; homovect only
+(check-tail-exn type-exception? (lambda () (s32vector-set! v5 0 2147483648))) ;; homovect only
 (check-tail-exn range-exception? (lambda () (s32vector-set! v5 -1 0)))
 (check-tail-exn range-exception? (lambda () (s32vector-set! v5 2 0)))
 
@@ -342,16 +391,18 @@
 (check-tail-exn range-exception? (lambda () (s32vector-shrink! v5 3)))
 
 (check-tail-exn type-exception? (lambda () (s32vector-fill! bool 0)))
-(check-tail-exn type-exception? (lambda () (s32vector-fill! v5 bool)))
-(check-tail-exn type-exception? (lambda () (s32vector-fill! v5 -2147483649)))
-(check-tail-exn type-exception? (lambda () (s32vector-fill! v5 2147483648)))
+(check-tail-exn type-exception? (lambda () (s32vector-fill! v5 0 bool)))
+(check-tail-exn type-exception? (lambda () (s32vector-fill! v5 0 0 bool)))
+(check-tail-exn type-exception? (lambda () (s32vector-fill! v5 bool))) ;; homovect only
+(check-tail-exn type-exception? (lambda () (s32vector-fill! v5 -2147483649))) ;; homovect only
+(check-tail-exn type-exception? (lambda () (s32vector-fill! v5 2147483648))) ;; homovect only
 
 (check-tail-exn type-exception? (lambda () (subs32vector-fill! bool 0 0 0)))
 (check-tail-exn type-exception? (lambda () (subs32vector-fill! v5 bool 0 0)))
 (check-tail-exn type-exception? (lambda () (subs32vector-fill! v5 0 bool 0)))
-(check-tail-exn type-exception? (lambda () (subs32vector-fill! v5 0 0 bool)))
-(check-tail-exn type-exception? (lambda () (subs32vector-fill! v5 0 0 -2147483649)))
-(check-tail-exn type-exception? (lambda () (subs32vector-fill! v5 0 0 2147483648)))
+(check-tail-exn type-exception? (lambda () (subs32vector-fill! v5 0 0 bool))) ;; homovect only
+(check-tail-exn type-exception? (lambda () (subs32vector-fill! v5 0 0 -2147483649))) ;; homovect only
+(check-tail-exn type-exception? (lambda () (subs32vector-fill! v5 0 0 2147483648))) ;; homovect only
 (check-tail-exn range-exception? (lambda () (subs32vector-fill! v5 -1 0 0)))
 (check-tail-exn range-exception? (lambda () (subs32vector-fill! v5 3 0 0)))
 (check-tail-exn range-exception? (lambda () (subs32vector-fill! v5 0 -1 0)))
@@ -369,6 +420,18 @@
 (check-tail-exn range-exception? (lambda () (subs32vector-move! v5 0 0 v5 -1)))
 (check-tail-exn range-exception? (lambda () (subs32vector-move! v5 0 0 v5 3)))
 
+(check-tail-exn type-exception? (lambda () (s32vector-copy! v5 0 bool 0 0)))
+(check-tail-exn type-exception? (lambda () (s32vector-copy! v5 0 v5 bool 0)))
+(check-tail-exn type-exception? (lambda () (s32vector-copy! v5 0 v5 0 bool)))
+(check-tail-exn type-exception? (lambda () (s32vector-copy! bool 0 v5 0 0)))
+(check-tail-exn type-exception? (lambda () (s32vector-copy! v5 bool v5 0 0)))
+(check-tail-exn range-exception? (lambda () (s32vector-copy! v5 0 v5 -1 0)))
+(check-tail-exn range-exception? (lambda () (s32vector-copy! v5 0 v5 3 0)))
+(check-tail-exn range-exception? (lambda () (s32vector-copy! v5 0 v5 0 -1)))
+(check-tail-exn range-exception? (lambda () (s32vector-copy! v5 0 v5 0 3)))
+(check-tail-exn range-exception? (lambda () (s32vector-copy! v5 -1 v5 0 0)))
+(check-tail-exn range-exception? (lambda () (s32vector-copy! v5 3 v5 0 0)))
+
 (check-tail-exn wrong-number-of-arguments-exception? (lambda () (make-s32vector)))
 (check-tail-exn wrong-number-of-arguments-exception? (lambda () (make-s32vector 11 22 33)))
 
@@ -379,7 +442,7 @@
 (check-tail-exn wrong-number-of-arguments-exception? (lambda () (s32vector-length bool bool)))
 
 (check-tail-exn wrong-number-of-arguments-exception? (lambda () (s32vector->list)))
-(check-tail-exn wrong-number-of-arguments-exception? (lambda () (s32vector->list v1 v1)))
+(check-tail-exn wrong-number-of-arguments-exception? (lambda () (s32vector->list v1 0 0 0)))
 
 (check-tail-exn wrong-number-of-arguments-exception? (lambda () (list->s32vector)))
 (check-tail-exn wrong-number-of-arguments-exception? (lambda () (list->s32vector '() '())))
@@ -388,7 +451,7 @@
 (check-tail-exn wrong-number-of-arguments-exception? (lambda () (append-s32vectors '() '())))
 
 (check-tail-exn wrong-number-of-arguments-exception? (lambda () (s32vector-copy)))
-(check-tail-exn wrong-number-of-arguments-exception? (lambda () (s32vector-copy v1 v1)))
+(check-tail-exn wrong-number-of-arguments-exception? (lambda () (s32vector-copy v1 0 0 0)))
 
 (check-tail-exn wrong-number-of-arguments-exception? (lambda () (subs32vector)))
 (check-tail-exn wrong-number-of-arguments-exception? (lambda () (subs32vector v1)))
@@ -410,7 +473,7 @@
 
 (check-tail-exn wrong-number-of-arguments-exception? (lambda () (s32vector-fill!)))
 (check-tail-exn wrong-number-of-arguments-exception? (lambda () (s32vector-fill! v9)))
-(check-tail-exn wrong-number-of-arguments-exception? (lambda () (s32vector-fill! v9 0 0)))
+(check-tail-exn wrong-number-of-arguments-exception? (lambda () (s32vector-fill! v9 0 0 0 0)))
 
 (check-tail-exn wrong-number-of-arguments-exception? (lambda () (subs32vector-fill!)))
 (check-tail-exn wrong-number-of-arguments-exception? (lambda () (subs32vector-fill! v9)))
@@ -425,4 +488,9 @@
 (check-tail-exn wrong-number-of-arguments-exception? (lambda () (subs32vector-move! v9 0 0 v9)))
 (check-tail-exn wrong-number-of-arguments-exception? (lambda () (subs32vector-move! v9 0 0 v9 0 0)))
 
+(check-tail-exn wrong-number-of-arguments-exception? (lambda () (s32vector-copy!)))
+(check-tail-exn wrong-number-of-arguments-exception? (lambda () (s32vector-copy! v9)))
+(check-tail-exn wrong-number-of-arguments-exception? (lambda () (s32vector-copy! v9 0)))
+(check-tail-exn wrong-number-of-arguments-exception? (lambda () (s32vector-copy! v9 0 v9 0 0 0)))
+
 (check-tail-exn range-exception? (lambda () (make-s32vector (expt 2 64))))
