@@ -101,6 +101,12 @@
 (check-eqv? (##u32vector-ref v5 0) 4294967295)
 (check-eqv? (##u32vector-ref v5 1) 4294967295)
 
+(check-equal? (##u32vector-set v2 1 99) '#u32(0 99 0 1 4294967295))
+(check-equal? v2 '#u32(0 4294967295 0 1 4294967295))
+(check-equal? (##u32vector-set v4 1 99) '#u32(0 99))
+(check-equal? (##u32vector-set v5 1 99) '#u32(4294967295 99))
+(check-equal? (##u32vector-set '#u32(11 22 33) 0 99) '#u32(99 22 33))
+
 (check-eq? (##u32vector-set! v2 1 99) v2)
 (check-eq? (##u32vector-set! v3 1 99) v3)
 (check-eq? (##u32vector-set! v4 1 99) v4)
@@ -198,6 +204,10 @@
 
 (check-equal? (u32vector->list '#u32()) '())
 (check-equal? (u32vector->list v6) '(0 4294967295 0 1 4294967295))
+(check-equal? (u32vector->list v6 0) '(0 4294967295 0 1 4294967295))
+(check-equal? (u32vector->list v6 2) '(0 1 4294967295))
+(check-equal? (u32vector->list v6 2 4) '(0 1))
+(check-equal? (u32vector->list v6 0 5) '(0 4294967295 0 1 4294967295))
 (check-equal? (u32vector->list v7) '(0 0))
 
 (check-equal? (list->u32vector '()) '#u32())
@@ -213,6 +223,14 @@
 
 (check-equal? (u32vector-copy '#u32()) '#u32())
 (check-equal? (u32vector-copy v6) v6)
+(check-equal? (u32vector-copy v6 0) v6)
+(check-equal? (u32vector-copy v6 2) '#u32(0 1 4294967295))
+(check-equal? (u32vector-copy v6 0 0) '#u32())
+(check-equal? (u32vector-copy v6 4 4) '#u32())
+(check-equal? (u32vector-copy v6 0 2) '#u32(0 4294967295))
+(check-equal? (u32vector-copy v6 2 4) '#u32(0 1))
+(check-equal? (u32vector-copy v6 4 5) '#u32(4294967295))
+(check-equal? (u32vector-copy v6 0 5) v6)
 
 (check-equal? (subu32vector v6 0 0) '#u32())
 (check-equal? (subu32vector v6 4 4) '#u32())
@@ -239,6 +257,12 @@
 (check-eqv? (u32vector-ref v9 0) 4294967295)
 (check-eqv? (u32vector-ref v9 1) 4294967295)
 
+(check-equal? (u32vector-set v6 1 99) '#u32(0 99 0 1 4294967295))
+(check-equal? v6 '#u32(0 4294967295 0 1 4294967295))
+(check-equal? (u32vector-set v8 1 99) '#u32(0 99))
+(check-equal? (u32vector-set v9 1 99) '#u32(4294967295 99))
+(check-equal? (u32vector-set '#u32(11 22 33) 0 99) '#u32(99 22 33))
+
 (check-eq? (u32vector-set! v6 1 99) (void))
 (check-eq? (u32vector-set! v7 1 99) (void))
 (check-eq? (u32vector-set! v8 1 99) (void))
@@ -275,6 +299,12 @@
 (check-eq? (u32vector-fill! v6 4294967295) (void))
 (check-equal? v6 '#u32(4294967295 4294967295 4294967295))
 
+(check-eq? (u32vector-fill! v6 3 1) (void))
+(check-equal? v6 '#u32(4294967295 3 3))
+
+(check-eq? (u32vector-fill! v6 99 0 2) (void))
+(check-equal? v6 '#u32(99 99 3))
+
 (check-eq? (subu32vector-fill! v6 0 3 9) (void))
 (check-equal? v6 '#u32(9 9 9))
 
@@ -290,15 +320,24 @@
 (check-eq? (subu32vector-move! v9 0 2 v6 1) (void))
 (check-equal? v6 '#u32(4294967295 4294967295 99))
 
-(check-tail-exn type-exception? (lambda () (u32vector 11 bool 22)))
-(check-tail-exn type-exception? (lambda () (u32vector 11 -1 22)))
-(check-tail-exn type-exception? (lambda () (u32vector 11 4294967296 22)))
+(check-eq? (u32vector-copy! v6 0 '#u32(11 22 33)) (void))
+(check-equal? v6 '#u32(11 22 33))
+
+(check-eq? (u32vector-copy! v6 2 '#u32(33 44) 1) (void))
+(check-equal? v6 '#u32(11 22 44))
+
+(check-eq? (u32vector-copy! v6 1 '#u32(55 66 77 88) 0 2) (void))
+(check-equal? v6 '#u32(11 55 66))
+
+(check-tail-exn type-exception? (lambda () (u32vector 11 bool 22))) ;; homovect only
+(check-tail-exn type-exception? (lambda () (u32vector 11 -1 22))) ;; homovect only
+(check-tail-exn type-exception? (lambda () (u32vector 11 4294967296 22))) ;; homovect only
 
 (check-tail-exn type-exception? (lambda () (make-u32vector bool)))
 (check-tail-exn type-exception? (lambda () (make-u32vector bool 11)))
-(check-tail-exn type-exception? (lambda () (make-u32vector 11 bool)))
-(check-tail-exn type-exception? (lambda () (make-u32vector 11 -1)))
-(check-tail-exn type-exception? (lambda () (make-u32vector 11 4294967296)))
+(check-tail-exn type-exception? (lambda () (make-u32vector 11 bool))) ;; homovect only
+(check-tail-exn type-exception? (lambda () (make-u32vector 11 -1))) ;; homovect only
+(check-tail-exn type-exception? (lambda () (make-u32vector 11 4294967296))) ;; homovect only
 (check-tail-exn range-exception? (lambda () (make-u32vector -1 0)))
 
 (check-tail-exn type-exception? (lambda () (u32vector-length bool)))
@@ -315,6 +354,8 @@
 (check-tail-exn type-exception? (lambda () (append-u32vectors '(1 2 3))))
 
 (check-tail-exn type-exception? (lambda () (u32vector-copy bool)))
+(check-tail-exn type-exception? (lambda () (u32vector-copy v9 bool)))
+(check-tail-exn type-exception? (lambda () (u32vector-copy v9 0 bool)))
 
 (check-tail-exn type-exception? (lambda () (subu32vector bool 0 0)))
 (check-tail-exn type-exception? (lambda () (subu32vector v9 bool 0)))
@@ -329,11 +370,19 @@
 (check-tail-exn range-exception? (lambda () (u32vector-ref v5 -1)))
 (check-tail-exn range-exception? (lambda () (u32vector-ref v5 2)))
 
+(check-tail-exn type-exception? (lambda () (u32vector-set bool 0 11)))
+(check-tail-exn type-exception? (lambda () (u32vector-set v5 bool 11)))
+(check-tail-exn type-exception? (lambda () (u32vector-set v5 0 bool))) ;; homovect only
+(check-tail-exn type-exception? (lambda () (u32vector-set v5 0 -1))) ;; homovect only
+(check-tail-exn type-exception? (lambda () (u32vector-set v5 0 4294967296))) ;; homovect only
+(check-tail-exn range-exception? (lambda () (u32vector-set v5 -1 0)))
+(check-tail-exn range-exception? (lambda () (u32vector-set v5 2 0)))
+
 (check-tail-exn type-exception? (lambda () (u32vector-set! bool 0 11)))
 (check-tail-exn type-exception? (lambda () (u32vector-set! v5 bool 11)))
-(check-tail-exn type-exception? (lambda () (u32vector-set! v5 0 bool)))
-(check-tail-exn type-exception? (lambda () (u32vector-set! v5 0 -1)))
-(check-tail-exn type-exception? (lambda () (u32vector-set! v5 0 4294967296)))
+(check-tail-exn type-exception? (lambda () (u32vector-set! v5 0 bool))) ;; homovect only
+(check-tail-exn type-exception? (lambda () (u32vector-set! v5 0 -1))) ;; homovect only
+(check-tail-exn type-exception? (lambda () (u32vector-set! v5 0 4294967296))) ;; homovect only
 (check-tail-exn range-exception? (lambda () (u32vector-set! v5 -1 0)))
 (check-tail-exn range-exception? (lambda () (u32vector-set! v5 2 0)))
 
@@ -342,16 +391,18 @@
 (check-tail-exn range-exception? (lambda () (u32vector-shrink! v5 3)))
 
 (check-tail-exn type-exception? (lambda () (u32vector-fill! bool 0)))
-(check-tail-exn type-exception? (lambda () (u32vector-fill! v5 bool)))
-(check-tail-exn type-exception? (lambda () (u32vector-fill! v5 -1)))
-(check-tail-exn type-exception? (lambda () (u32vector-fill! v5 4294967296)))
+(check-tail-exn type-exception? (lambda () (u32vector-fill! v5 0 bool)))
+(check-tail-exn type-exception? (lambda () (u32vector-fill! v5 0 0 bool)))
+(check-tail-exn type-exception? (lambda () (u32vector-fill! v5 bool))) ;; homovect only
+(check-tail-exn type-exception? (lambda () (u32vector-fill! v5 -1))) ;; homovect only
+(check-tail-exn type-exception? (lambda () (u32vector-fill! v5 4294967296))) ;; homovect only
 
 (check-tail-exn type-exception? (lambda () (subu32vector-fill! bool 0 0 0)))
 (check-tail-exn type-exception? (lambda () (subu32vector-fill! v5 bool 0 0)))
 (check-tail-exn type-exception? (lambda () (subu32vector-fill! v5 0 bool 0)))
-(check-tail-exn type-exception? (lambda () (subu32vector-fill! v5 0 0 bool)))
-(check-tail-exn type-exception? (lambda () (subu32vector-fill! v5 0 0 -1)))
-(check-tail-exn type-exception? (lambda () (subu32vector-fill! v5 0 0 4294967296)))
+(check-tail-exn type-exception? (lambda () (subu32vector-fill! v5 0 0 bool))) ;; homovect only
+(check-tail-exn type-exception? (lambda () (subu32vector-fill! v5 0 0 -1))) ;; homovect only
+(check-tail-exn type-exception? (lambda () (subu32vector-fill! v5 0 0 4294967296))) ;; homovect only
 (check-tail-exn range-exception? (lambda () (subu32vector-fill! v5 -1 0 0)))
 (check-tail-exn range-exception? (lambda () (subu32vector-fill! v5 3 0 0)))
 (check-tail-exn range-exception? (lambda () (subu32vector-fill! v5 0 -1 0)))
@@ -369,6 +420,18 @@
 (check-tail-exn range-exception? (lambda () (subu32vector-move! v5 0 0 v5 -1)))
 (check-tail-exn range-exception? (lambda () (subu32vector-move! v5 0 0 v5 3)))
 
+(check-tail-exn type-exception? (lambda () (u32vector-copy! v5 0 bool 0 0)))
+(check-tail-exn type-exception? (lambda () (u32vector-copy! v5 0 v5 bool 0)))
+(check-tail-exn type-exception? (lambda () (u32vector-copy! v5 0 v5 0 bool)))
+(check-tail-exn type-exception? (lambda () (u32vector-copy! bool 0 v5 0 0)))
+(check-tail-exn type-exception? (lambda () (u32vector-copy! v5 bool v5 0 0)))
+(check-tail-exn range-exception? (lambda () (u32vector-copy! v5 0 v5 -1 0)))
+(check-tail-exn range-exception? (lambda () (u32vector-copy! v5 0 v5 3 0)))
+(check-tail-exn range-exception? (lambda () (u32vector-copy! v5 0 v5 0 -1)))
+(check-tail-exn range-exception? (lambda () (u32vector-copy! v5 0 v5 0 3)))
+(check-tail-exn range-exception? (lambda () (u32vector-copy! v5 -1 v5 0 0)))
+(check-tail-exn range-exception? (lambda () (u32vector-copy! v5 3 v5 0 0)))
+
 (check-tail-exn wrong-number-of-arguments-exception? (lambda () (make-u32vector)))
 (check-tail-exn wrong-number-of-arguments-exception? (lambda () (make-u32vector 11 22 33)))
 
@@ -379,7 +442,7 @@
 (check-tail-exn wrong-number-of-arguments-exception? (lambda () (u32vector-length bool bool)))
 
 (check-tail-exn wrong-number-of-arguments-exception? (lambda () (u32vector->list)))
-(check-tail-exn wrong-number-of-arguments-exception? (lambda () (u32vector->list v1 v1)))
+(check-tail-exn wrong-number-of-arguments-exception? (lambda () (u32vector->list v1 0 0 0)))
 
 (check-tail-exn wrong-number-of-arguments-exception? (lambda () (list->u32vector)))
 (check-tail-exn wrong-number-of-arguments-exception? (lambda () (list->u32vector '() '())))
@@ -388,7 +451,7 @@
 (check-tail-exn wrong-number-of-arguments-exception? (lambda () (append-u32vectors '() '())))
 
 (check-tail-exn wrong-number-of-arguments-exception? (lambda () (u32vector-copy)))
-(check-tail-exn wrong-number-of-arguments-exception? (lambda () (u32vector-copy v1 v1)))
+(check-tail-exn wrong-number-of-arguments-exception? (lambda () (u32vector-copy v1 0 0 0)))
 
 (check-tail-exn wrong-number-of-arguments-exception? (lambda () (subu32vector)))
 (check-tail-exn wrong-number-of-arguments-exception? (lambda () (subu32vector v1)))
@@ -410,7 +473,7 @@
 
 (check-tail-exn wrong-number-of-arguments-exception? (lambda () (u32vector-fill!)))
 (check-tail-exn wrong-number-of-arguments-exception? (lambda () (u32vector-fill! v9)))
-(check-tail-exn wrong-number-of-arguments-exception? (lambda () (u32vector-fill! v9 0 0)))
+(check-tail-exn wrong-number-of-arguments-exception? (lambda () (u32vector-fill! v9 0 0 0 0)))
 
 (check-tail-exn wrong-number-of-arguments-exception? (lambda () (subu32vector-fill!)))
 (check-tail-exn wrong-number-of-arguments-exception? (lambda () (subu32vector-fill! v9)))
@@ -425,4 +488,9 @@
 (check-tail-exn wrong-number-of-arguments-exception? (lambda () (subu32vector-move! v9 0 0 v9)))
 (check-tail-exn wrong-number-of-arguments-exception? (lambda () (subu32vector-move! v9 0 0 v9 0 0)))
 
+(check-tail-exn wrong-number-of-arguments-exception? (lambda () (u32vector-copy!)))
+(check-tail-exn wrong-number-of-arguments-exception? (lambda () (u32vector-copy! v9)))
+(check-tail-exn wrong-number-of-arguments-exception? (lambda () (u32vector-copy! v9 0)))
+(check-tail-exn wrong-number-of-arguments-exception? (lambda () (u32vector-copy! v9 0 v9 0 0 0)))
+
 (check-tail-exn range-exception? (lambda () (make-u32vector (expt 2 64))))
