@@ -101,6 +101,12 @@
 (check-eqv? (##u64vector-ref v5 0) 18446744073709551615)
 (check-eqv? (##u64vector-ref v5 1) 18446744073709551615)
 
+(check-equal? (##u64vector-set v2 1 99) '#u64(0 99 0 1 18446744073709551615))
+(check-equal? v2 '#u64(0 18446744073709551615 0 1 18446744073709551615))
+(check-equal? (##u64vector-set v4 1 99) '#u64(0 99))
+(check-equal? (##u64vector-set v5 1 99) '#u64(18446744073709551615 99))
+(check-equal? (##u64vector-set '#u64(11 22 33) 0 99) '#u64(99 22 33))
+
 (check-eq? (##u64vector-set! v2 1 99) v2)
 (check-eq? (##u64vector-set! v3 1 99) v3)
 (check-eq? (##u64vector-set! v4 1 99) v4)
@@ -198,6 +204,10 @@
 
 (check-equal? (u64vector->list '#u64()) '())
 (check-equal? (u64vector->list v6) '(0 18446744073709551615 0 1 18446744073709551615))
+(check-equal? (u64vector->list v6 0) '(0 18446744073709551615 0 1 18446744073709551615))
+(check-equal? (u64vector->list v6 2) '(0 1 18446744073709551615))
+(check-equal? (u64vector->list v6 2 4) '(0 1))
+(check-equal? (u64vector->list v6 0 5) '(0 18446744073709551615 0 1 18446744073709551615))
 (check-equal? (u64vector->list v7) '(0 0))
 
 (check-equal? (list->u64vector '()) '#u64())
@@ -213,6 +223,14 @@
 
 (check-equal? (u64vector-copy '#u64()) '#u64())
 (check-equal? (u64vector-copy v6) v6)
+(check-equal? (u64vector-copy v6 0) v6)
+(check-equal? (u64vector-copy v6 2) '#u64(0 1 18446744073709551615))
+(check-equal? (u64vector-copy v6 0 0) '#u64())
+(check-equal? (u64vector-copy v6 4 4) '#u64())
+(check-equal? (u64vector-copy v6 0 2) '#u64(0 18446744073709551615))
+(check-equal? (u64vector-copy v6 2 4) '#u64(0 1))
+(check-equal? (u64vector-copy v6 4 5) '#u64(18446744073709551615))
+(check-equal? (u64vector-copy v6 0 5) v6)
 
 (check-equal? (subu64vector v6 0 0) '#u64())
 (check-equal? (subu64vector v6 4 4) '#u64())
@@ -239,6 +257,12 @@
 (check-eqv? (u64vector-ref v9 0) 18446744073709551615)
 (check-eqv? (u64vector-ref v9 1) 18446744073709551615)
 
+(check-equal? (u64vector-set v6 1 99) '#u64(0 99 0 1 18446744073709551615))
+(check-equal? v6 '#u64(0 18446744073709551615 0 1 18446744073709551615))
+(check-equal? (u64vector-set v8 1 99) '#u64(0 99))
+(check-equal? (u64vector-set v9 1 99) '#u64(18446744073709551615 99))
+(check-equal? (u64vector-set '#u64(11 22 33) 0 99) '#u64(99 22 33))
+
 (check-eq? (u64vector-set! v6 1 99) (void))
 (check-eq? (u64vector-set! v7 1 99) (void))
 (check-eq? (u64vector-set! v8 1 99) (void))
@@ -275,6 +299,12 @@
 (check-eq? (u64vector-fill! v6 18446744073709551615) (void))
 (check-equal? v6 '#u64(18446744073709551615 18446744073709551615 18446744073709551615))
 
+(check-eq? (u64vector-fill! v6 3 1) (void))
+(check-equal? v6 '#u64(18446744073709551615 3 3))
+
+(check-eq? (u64vector-fill! v6 99 0 2) (void))
+(check-equal? v6 '#u64(99 99 3))
+
 (check-eq? (subu64vector-fill! v6 0 3 9) (void))
 (check-equal? v6 '#u64(9 9 9))
 
@@ -290,15 +320,24 @@
 (check-eq? (subu64vector-move! v9 0 2 v6 1) (void))
 (check-equal? v6 '#u64(18446744073709551615 18446744073709551615 99))
 
-(check-tail-exn type-exception? (lambda () (u64vector 11 bool 22)))
-(check-tail-exn type-exception? (lambda () (u64vector 11 -1 22)))
-(check-tail-exn type-exception? (lambda () (u64vector 11 18446744073709551616 22)))
+(check-eq? (u64vector-copy! v6 0 '#u64(11 22 33)) (void))
+(check-equal? v6 '#u64(11 22 33))
+
+(check-eq? (u64vector-copy! v6 2 '#u64(33 44) 1) (void))
+(check-equal? v6 '#u64(11 22 44))
+
+(check-eq? (u64vector-copy! v6 1 '#u64(55 66 77 88) 0 2) (void))
+(check-equal? v6 '#u64(11 55 66))
+
+(check-tail-exn type-exception? (lambda () (u64vector 11 bool 22))) ;; homovect only
+(check-tail-exn type-exception? (lambda () (u64vector 11 -1 22))) ;; homovect only
+(check-tail-exn type-exception? (lambda () (u64vector 11 18446744073709551616 22))) ;; homovect only
 
 (check-tail-exn type-exception? (lambda () (make-u64vector bool)))
 (check-tail-exn type-exception? (lambda () (make-u64vector bool 11)))
-(check-tail-exn type-exception? (lambda () (make-u64vector 11 bool)))
-(check-tail-exn type-exception? (lambda () (make-u64vector 11 -1)))
-(check-tail-exn type-exception? (lambda () (make-u64vector 11 18446744073709551616)))
+(check-tail-exn type-exception? (lambda () (make-u64vector 11 bool))) ;; homovect only
+(check-tail-exn type-exception? (lambda () (make-u64vector 11 -1))) ;; homovect only
+(check-tail-exn type-exception? (lambda () (make-u64vector 11 18446744073709551616))) ;; homovect only
 (check-tail-exn range-exception? (lambda () (make-u64vector -1 0)))
 
 (check-tail-exn type-exception? (lambda () (u64vector-length bool)))
@@ -315,6 +354,8 @@
 (check-tail-exn type-exception? (lambda () (append-u64vectors '(1 2 3))))
 
 (check-tail-exn type-exception? (lambda () (u64vector-copy bool)))
+(check-tail-exn type-exception? (lambda () (u64vector-copy v9 bool)))
+(check-tail-exn type-exception? (lambda () (u64vector-copy v9 0 bool)))
 
 (check-tail-exn type-exception? (lambda () (subu64vector bool 0 0)))
 (check-tail-exn type-exception? (lambda () (subu64vector v9 bool 0)))
@@ -329,11 +370,19 @@
 (check-tail-exn range-exception? (lambda () (u64vector-ref v5 -1)))
 (check-tail-exn range-exception? (lambda () (u64vector-ref v5 2)))
 
+(check-tail-exn type-exception? (lambda () (u64vector-set bool 0 11)))
+(check-tail-exn type-exception? (lambda () (u64vector-set v5 bool 11)))
+(check-tail-exn type-exception? (lambda () (u64vector-set v5 0 bool))) ;; homovect only
+(check-tail-exn type-exception? (lambda () (u64vector-set v5 0 -1))) ;; homovect only
+(check-tail-exn type-exception? (lambda () (u64vector-set v5 0 18446744073709551616))) ;; homovect only
+(check-tail-exn range-exception? (lambda () (u64vector-set v5 -1 0)))
+(check-tail-exn range-exception? (lambda () (u64vector-set v5 2 0)))
+
 (check-tail-exn type-exception? (lambda () (u64vector-set! bool 0 11)))
 (check-tail-exn type-exception? (lambda () (u64vector-set! v5 bool 11)))
-(check-tail-exn type-exception? (lambda () (u64vector-set! v5 0 bool)))
-(check-tail-exn type-exception? (lambda () (u64vector-set! v5 0 -1)))
-(check-tail-exn type-exception? (lambda () (u64vector-set! v5 0 18446744073709551616)))
+(check-tail-exn type-exception? (lambda () (u64vector-set! v5 0 bool))) ;; homovect only
+(check-tail-exn type-exception? (lambda () (u64vector-set! v5 0 -1))) ;; homovect only
+(check-tail-exn type-exception? (lambda () (u64vector-set! v5 0 18446744073709551616))) ;; homovect only
 (check-tail-exn range-exception? (lambda () (u64vector-set! v5 -1 0)))
 (check-tail-exn range-exception? (lambda () (u64vector-set! v5 2 0)))
 
@@ -342,16 +391,18 @@
 (check-tail-exn range-exception? (lambda () (u64vector-shrink! v5 3)))
 
 (check-tail-exn type-exception? (lambda () (u64vector-fill! bool 0)))
-(check-tail-exn type-exception? (lambda () (u64vector-fill! v5 bool)))
-(check-tail-exn type-exception? (lambda () (u64vector-fill! v5 -1)))
-(check-tail-exn type-exception? (lambda () (u64vector-fill! v5 18446744073709551616)))
+(check-tail-exn type-exception? (lambda () (u64vector-fill! v5 0 bool)))
+(check-tail-exn type-exception? (lambda () (u64vector-fill! v5 0 0 bool)))
+(check-tail-exn type-exception? (lambda () (u64vector-fill! v5 bool))) ;; homovect only
+(check-tail-exn type-exception? (lambda () (u64vector-fill! v5 -1))) ;; homovect only
+(check-tail-exn type-exception? (lambda () (u64vector-fill! v5 18446744073709551616))) ;; homovect only
 
 (check-tail-exn type-exception? (lambda () (subu64vector-fill! bool 0 0 0)))
 (check-tail-exn type-exception? (lambda () (subu64vector-fill! v5 bool 0 0)))
 (check-tail-exn type-exception? (lambda () (subu64vector-fill! v5 0 bool 0)))
-(check-tail-exn type-exception? (lambda () (subu64vector-fill! v5 0 0 bool)))
-(check-tail-exn type-exception? (lambda () (subu64vector-fill! v5 0 0 -1)))
-(check-tail-exn type-exception? (lambda () (subu64vector-fill! v5 0 0 18446744073709551616)))
+(check-tail-exn type-exception? (lambda () (subu64vector-fill! v5 0 0 bool))) ;; homovect only
+(check-tail-exn type-exception? (lambda () (subu64vector-fill! v5 0 0 -1))) ;; homovect only
+(check-tail-exn type-exception? (lambda () (subu64vector-fill! v5 0 0 18446744073709551616))) ;; homovect only
 (check-tail-exn range-exception? (lambda () (subu64vector-fill! v5 -1 0 0)))
 (check-tail-exn range-exception? (lambda () (subu64vector-fill! v5 3 0 0)))
 (check-tail-exn range-exception? (lambda () (subu64vector-fill! v5 0 -1 0)))
@@ -369,6 +420,18 @@
 (check-tail-exn range-exception? (lambda () (subu64vector-move! v5 0 0 v5 -1)))
 (check-tail-exn range-exception? (lambda () (subu64vector-move! v5 0 0 v5 3)))
 
+(check-tail-exn type-exception? (lambda () (u64vector-copy! v5 0 bool 0 0)))
+(check-tail-exn type-exception? (lambda () (u64vector-copy! v5 0 v5 bool 0)))
+(check-tail-exn type-exception? (lambda () (u64vector-copy! v5 0 v5 0 bool)))
+(check-tail-exn type-exception? (lambda () (u64vector-copy! bool 0 v5 0 0)))
+(check-tail-exn type-exception? (lambda () (u64vector-copy! v5 bool v5 0 0)))
+(check-tail-exn range-exception? (lambda () (u64vector-copy! v5 0 v5 -1 0)))
+(check-tail-exn range-exception? (lambda () (u64vector-copy! v5 0 v5 3 0)))
+(check-tail-exn range-exception? (lambda () (u64vector-copy! v5 0 v5 0 -1)))
+(check-tail-exn range-exception? (lambda () (u64vector-copy! v5 0 v5 0 3)))
+(check-tail-exn range-exception? (lambda () (u64vector-copy! v5 -1 v5 0 0)))
+(check-tail-exn range-exception? (lambda () (u64vector-copy! v5 3 v5 0 0)))
+
 (check-tail-exn wrong-number-of-arguments-exception? (lambda () (make-u64vector)))
 (check-tail-exn wrong-number-of-arguments-exception? (lambda () (make-u64vector 11 22 33)))
 
@@ -379,7 +442,7 @@
 (check-tail-exn wrong-number-of-arguments-exception? (lambda () (u64vector-length bool bool)))
 
 (check-tail-exn wrong-number-of-arguments-exception? (lambda () (u64vector->list)))
-(check-tail-exn wrong-number-of-arguments-exception? (lambda () (u64vector->list v1 v1)))
+(check-tail-exn wrong-number-of-arguments-exception? (lambda () (u64vector->list v1 0 0 0)))
 
 (check-tail-exn wrong-number-of-arguments-exception? (lambda () (list->u64vector)))
 (check-tail-exn wrong-number-of-arguments-exception? (lambda () (list->u64vector '() '())))
@@ -388,7 +451,7 @@
 (check-tail-exn wrong-number-of-arguments-exception? (lambda () (append-u64vectors '() '())))
 
 (check-tail-exn wrong-number-of-arguments-exception? (lambda () (u64vector-copy)))
-(check-tail-exn wrong-number-of-arguments-exception? (lambda () (u64vector-copy v1 v1)))
+(check-tail-exn wrong-number-of-arguments-exception? (lambda () (u64vector-copy v1 0 0 0)))
 
 (check-tail-exn wrong-number-of-arguments-exception? (lambda () (subu64vector)))
 (check-tail-exn wrong-number-of-arguments-exception? (lambda () (subu64vector v1)))
@@ -410,7 +473,7 @@
 
 (check-tail-exn wrong-number-of-arguments-exception? (lambda () (u64vector-fill!)))
 (check-tail-exn wrong-number-of-arguments-exception? (lambda () (u64vector-fill! v9)))
-(check-tail-exn wrong-number-of-arguments-exception? (lambda () (u64vector-fill! v9 0 0)))
+(check-tail-exn wrong-number-of-arguments-exception? (lambda () (u64vector-fill! v9 0 0 0 0)))
 
 (check-tail-exn wrong-number-of-arguments-exception? (lambda () (subu64vector-fill!)))
 (check-tail-exn wrong-number-of-arguments-exception? (lambda () (subu64vector-fill! v9)))
@@ -425,4 +488,9 @@
 (check-tail-exn wrong-number-of-arguments-exception? (lambda () (subu64vector-move! v9 0 0 v9)))
 (check-tail-exn wrong-number-of-arguments-exception? (lambda () (subu64vector-move! v9 0 0 v9 0 0)))
 
+(check-tail-exn wrong-number-of-arguments-exception? (lambda () (u64vector-copy!)))
+(check-tail-exn wrong-number-of-arguments-exception? (lambda () (u64vector-copy! v9)))
+(check-tail-exn wrong-number-of-arguments-exception? (lambda () (u64vector-copy! v9 0)))
+(check-tail-exn wrong-number-of-arguments-exception? (lambda () (u64vector-copy! v9 0 v9 0 0 0)))
+
 (check-tail-exn range-exception? (lambda () (make-u64vector (expt 2 64))))
