@@ -101,6 +101,12 @@
 (check-eqv? (##u8vector-ref v5 0) 255)
 (check-eqv? (##u8vector-ref v5 1) 255)
 
+(check-equal? (##u8vector-set v2 1 99) '#u8(0 99 0 1 255))
+(check-equal? v2 '#u8(0 255 0 1 255))
+(check-equal? (##u8vector-set v4 1 99) '#u8(0 99))
+(check-equal? (##u8vector-set v5 1 99) '#u8(255 99))
+(check-equal? (##u8vector-set '#u8(11 22 33) 0 99) '#u8(99 22 33))
+
 (check-eq? (##u8vector-set! v2 1 99) v2)
 (check-eq? (##u8vector-set! v3 1 99) v3)
 (check-eq? (##u8vector-set! v4 1 99) v4)
@@ -198,6 +204,10 @@
 
 (check-equal? (u8vector->list '#u8()) '())
 (check-equal? (u8vector->list v6) '(0 255 0 1 255))
+(check-equal? (u8vector->list v6 0) '(0 255 0 1 255))
+(check-equal? (u8vector->list v6 2) '(0 1 255))
+(check-equal? (u8vector->list v6 2 4) '(0 1))
+(check-equal? (u8vector->list v6 0 5) '(0 255 0 1 255))
 (check-equal? (u8vector->list v7) '(0 0))
 
 (check-equal? (list->u8vector '()) '#u8())
@@ -213,6 +223,14 @@
 
 (check-equal? (u8vector-copy '#u8()) '#u8())
 (check-equal? (u8vector-copy v6) v6)
+(check-equal? (u8vector-copy v6 0) v6)
+(check-equal? (u8vector-copy v6 2) '#u8(0 1 255))
+(check-equal? (u8vector-copy v6 0 0) '#u8())
+(check-equal? (u8vector-copy v6 4 4) '#u8())
+(check-equal? (u8vector-copy v6 0 2) '#u8(0 255))
+(check-equal? (u8vector-copy v6 2 4) '#u8(0 1))
+(check-equal? (u8vector-copy v6 4 5) '#u8(255))
+(check-equal? (u8vector-copy v6 0 5) v6)
 
 (check-equal? (subu8vector v6 0 0) '#u8())
 (check-equal? (subu8vector v6 4 4) '#u8())
@@ -239,6 +257,12 @@
 (check-eqv? (u8vector-ref v9 0) 255)
 (check-eqv? (u8vector-ref v9 1) 255)
 
+(check-equal? (u8vector-set v6 1 99) '#u8(0 99 0 1 255))
+(check-equal? v6 '#u8(0 255 0 1 255))
+(check-equal? (u8vector-set v8 1 99) '#u8(0 99))
+(check-equal? (u8vector-set v9 1 99) '#u8(255 99))
+(check-equal? (u8vector-set '#u8(11 22 33) 0 99) '#u8(99 22 33))
+
 (check-eq? (u8vector-set! v6 1 99) (void))
 (check-eq? (u8vector-set! v7 1 99) (void))
 (check-eq? (u8vector-set! v8 1 99) (void))
@@ -275,6 +299,12 @@
 (check-eq? (u8vector-fill! v6 255) (void))
 (check-equal? v6 '#u8(255 255 255))
 
+(check-eq? (u8vector-fill! v6 3 1) (void))
+(check-equal? v6 '#u8(255 3 3))
+
+(check-eq? (u8vector-fill! v6 99 0 2) (void))
+(check-equal? v6 '#u8(99 99 3))
+
 (check-eq? (subu8vector-fill! v6 0 3 9) (void))
 (check-equal? v6 '#u8(9 9 9))
 
@@ -290,15 +320,24 @@
 (check-eq? (subu8vector-move! v9 0 2 v6 1) (void))
 (check-equal? v6 '#u8(255 255 99))
 
-(check-tail-exn type-exception? (lambda () (u8vector 11 bool 22)))
-(check-tail-exn type-exception? (lambda () (u8vector 11 -1 22)))
-(check-tail-exn type-exception? (lambda () (u8vector 11 256 22)))
+(check-eq? (u8vector-copy! v6 0 '#u8(11 22 33)) (void))
+(check-equal? v6 '#u8(11 22 33))
+
+(check-eq? (u8vector-copy! v6 2 '#u8(33 44) 1) (void))
+(check-equal? v6 '#u8(11 22 44))
+
+(check-eq? (u8vector-copy! v6 1 '#u8(55 66 77 88) 0 2) (void))
+(check-equal? v6 '#u8(11 55 66))
+
+(check-tail-exn type-exception? (lambda () (u8vector 11 bool 22))) ;; homovect only
+(check-tail-exn type-exception? (lambda () (u8vector 11 -1 22))) ;; homovect only
+(check-tail-exn type-exception? (lambda () (u8vector 11 256 22))) ;; homovect only
 
 (check-tail-exn type-exception? (lambda () (make-u8vector bool)))
 (check-tail-exn type-exception? (lambda () (make-u8vector bool 11)))
-(check-tail-exn type-exception? (lambda () (make-u8vector 11 bool)))
-(check-tail-exn type-exception? (lambda () (make-u8vector 11 -1)))
-(check-tail-exn type-exception? (lambda () (make-u8vector 11 256)))
+(check-tail-exn type-exception? (lambda () (make-u8vector 11 bool))) ;; homovect only
+(check-tail-exn type-exception? (lambda () (make-u8vector 11 -1))) ;; homovect only
+(check-tail-exn type-exception? (lambda () (make-u8vector 11 256))) ;; homovect only
 (check-tail-exn range-exception? (lambda () (make-u8vector -1 0)))
 
 (check-tail-exn type-exception? (lambda () (u8vector-length bool)))
@@ -315,6 +354,8 @@
 (check-tail-exn type-exception? (lambda () (append-u8vectors '(1 2 3))))
 
 (check-tail-exn type-exception? (lambda () (u8vector-copy bool)))
+(check-tail-exn type-exception? (lambda () (u8vector-copy v9 bool)))
+(check-tail-exn type-exception? (lambda () (u8vector-copy v9 0 bool)))
 
 (check-tail-exn type-exception? (lambda () (subu8vector bool 0 0)))
 (check-tail-exn type-exception? (lambda () (subu8vector v9 bool 0)))
@@ -329,11 +370,19 @@
 (check-tail-exn range-exception? (lambda () (u8vector-ref v5 -1)))
 (check-tail-exn range-exception? (lambda () (u8vector-ref v5 2)))
 
+(check-tail-exn type-exception? (lambda () (u8vector-set bool 0 11)))
+(check-tail-exn type-exception? (lambda () (u8vector-set v5 bool 11)))
+(check-tail-exn type-exception? (lambda () (u8vector-set v5 0 bool))) ;; homovect only
+(check-tail-exn type-exception? (lambda () (u8vector-set v5 0 -1))) ;; homovect only
+(check-tail-exn type-exception? (lambda () (u8vector-set v5 0 256))) ;; homovect only
+(check-tail-exn range-exception? (lambda () (u8vector-set v5 -1 0)))
+(check-tail-exn range-exception? (lambda () (u8vector-set v5 2 0)))
+
 (check-tail-exn type-exception? (lambda () (u8vector-set! bool 0 11)))
 (check-tail-exn type-exception? (lambda () (u8vector-set! v5 bool 11)))
-(check-tail-exn type-exception? (lambda () (u8vector-set! v5 0 bool)))
-(check-tail-exn type-exception? (lambda () (u8vector-set! v5 0 -1)))
-(check-tail-exn type-exception? (lambda () (u8vector-set! v5 0 256)))
+(check-tail-exn type-exception? (lambda () (u8vector-set! v5 0 bool))) ;; homovect only
+(check-tail-exn type-exception? (lambda () (u8vector-set! v5 0 -1))) ;; homovect only
+(check-tail-exn type-exception? (lambda () (u8vector-set! v5 0 256))) ;; homovect only
 (check-tail-exn range-exception? (lambda () (u8vector-set! v5 -1 0)))
 (check-tail-exn range-exception? (lambda () (u8vector-set! v5 2 0)))
 
@@ -342,16 +391,18 @@
 (check-tail-exn range-exception? (lambda () (u8vector-shrink! v5 3)))
 
 (check-tail-exn type-exception? (lambda () (u8vector-fill! bool 0)))
-(check-tail-exn type-exception? (lambda () (u8vector-fill! v5 bool)))
-(check-tail-exn type-exception? (lambda () (u8vector-fill! v5 -1)))
-(check-tail-exn type-exception? (lambda () (u8vector-fill! v5 256)))
+(check-tail-exn type-exception? (lambda () (u8vector-fill! v5 0 bool)))
+(check-tail-exn type-exception? (lambda () (u8vector-fill! v5 0 0 bool)))
+(check-tail-exn type-exception? (lambda () (u8vector-fill! v5 bool))) ;; homovect only
+(check-tail-exn type-exception? (lambda () (u8vector-fill! v5 -1))) ;; homovect only
+(check-tail-exn type-exception? (lambda () (u8vector-fill! v5 256))) ;; homovect only
 
 (check-tail-exn type-exception? (lambda () (subu8vector-fill! bool 0 0 0)))
 (check-tail-exn type-exception? (lambda () (subu8vector-fill! v5 bool 0 0)))
 (check-tail-exn type-exception? (lambda () (subu8vector-fill! v5 0 bool 0)))
-(check-tail-exn type-exception? (lambda () (subu8vector-fill! v5 0 0 bool)))
-(check-tail-exn type-exception? (lambda () (subu8vector-fill! v5 0 0 -1)))
-(check-tail-exn type-exception? (lambda () (subu8vector-fill! v5 0 0 256)))
+(check-tail-exn type-exception? (lambda () (subu8vector-fill! v5 0 0 bool))) ;; homovect only
+(check-tail-exn type-exception? (lambda () (subu8vector-fill! v5 0 0 -1))) ;; homovect only
+(check-tail-exn type-exception? (lambda () (subu8vector-fill! v5 0 0 256))) ;; homovect only
 (check-tail-exn range-exception? (lambda () (subu8vector-fill! v5 -1 0 0)))
 (check-tail-exn range-exception? (lambda () (subu8vector-fill! v5 3 0 0)))
 (check-tail-exn range-exception? (lambda () (subu8vector-fill! v5 0 -1 0)))
@@ -369,6 +420,18 @@
 (check-tail-exn range-exception? (lambda () (subu8vector-move! v5 0 0 v5 -1)))
 (check-tail-exn range-exception? (lambda () (subu8vector-move! v5 0 0 v5 3)))
 
+(check-tail-exn type-exception? (lambda () (u8vector-copy! v5 0 bool 0 0)))
+(check-tail-exn type-exception? (lambda () (u8vector-copy! v5 0 v5 bool 0)))
+(check-tail-exn type-exception? (lambda () (u8vector-copy! v5 0 v5 0 bool)))
+(check-tail-exn type-exception? (lambda () (u8vector-copy! bool 0 v5 0 0)))
+(check-tail-exn type-exception? (lambda () (u8vector-copy! v5 bool v5 0 0)))
+(check-tail-exn range-exception? (lambda () (u8vector-copy! v5 0 v5 -1 0)))
+(check-tail-exn range-exception? (lambda () (u8vector-copy! v5 0 v5 3 0)))
+(check-tail-exn range-exception? (lambda () (u8vector-copy! v5 0 v5 0 -1)))
+(check-tail-exn range-exception? (lambda () (u8vector-copy! v5 0 v5 0 3)))
+(check-tail-exn range-exception? (lambda () (u8vector-copy! v5 -1 v5 0 0)))
+(check-tail-exn range-exception? (lambda () (u8vector-copy! v5 3 v5 0 0)))
+
 (check-tail-exn wrong-number-of-arguments-exception? (lambda () (make-u8vector)))
 (check-tail-exn wrong-number-of-arguments-exception? (lambda () (make-u8vector 11 22 33)))
 
@@ -379,7 +442,7 @@
 (check-tail-exn wrong-number-of-arguments-exception? (lambda () (u8vector-length bool bool)))
 
 (check-tail-exn wrong-number-of-arguments-exception? (lambda () (u8vector->list)))
-(check-tail-exn wrong-number-of-arguments-exception? (lambda () (u8vector->list v1 v1)))
+(check-tail-exn wrong-number-of-arguments-exception? (lambda () (u8vector->list v1 0 0 0)))
 
 (check-tail-exn wrong-number-of-arguments-exception? (lambda () (list->u8vector)))
 (check-tail-exn wrong-number-of-arguments-exception? (lambda () (list->u8vector '() '())))
@@ -388,7 +451,7 @@
 (check-tail-exn wrong-number-of-arguments-exception? (lambda () (append-u8vectors '() '())))
 
 (check-tail-exn wrong-number-of-arguments-exception? (lambda () (u8vector-copy)))
-(check-tail-exn wrong-number-of-arguments-exception? (lambda () (u8vector-copy v1 v1)))
+(check-tail-exn wrong-number-of-arguments-exception? (lambda () (u8vector-copy v1 0 0 0)))
 
 (check-tail-exn wrong-number-of-arguments-exception? (lambda () (subu8vector)))
 (check-tail-exn wrong-number-of-arguments-exception? (lambda () (subu8vector v1)))
@@ -410,7 +473,7 @@
 
 (check-tail-exn wrong-number-of-arguments-exception? (lambda () (u8vector-fill!)))
 (check-tail-exn wrong-number-of-arguments-exception? (lambda () (u8vector-fill! v9)))
-(check-tail-exn wrong-number-of-arguments-exception? (lambda () (u8vector-fill! v9 0 0)))
+(check-tail-exn wrong-number-of-arguments-exception? (lambda () (u8vector-fill! v9 0 0 0 0)))
 
 (check-tail-exn wrong-number-of-arguments-exception? (lambda () (subu8vector-fill!)))
 (check-tail-exn wrong-number-of-arguments-exception? (lambda () (subu8vector-fill! v9)))
@@ -425,4 +488,9 @@
 (check-tail-exn wrong-number-of-arguments-exception? (lambda () (subu8vector-move! v9 0 0 v9)))
 (check-tail-exn wrong-number-of-arguments-exception? (lambda () (subu8vector-move! v9 0 0 v9 0 0)))
 
+(check-tail-exn wrong-number-of-arguments-exception? (lambda () (u8vector-copy!)))
+(check-tail-exn wrong-number-of-arguments-exception? (lambda () (u8vector-copy! v9)))
+(check-tail-exn wrong-number-of-arguments-exception? (lambda () (u8vector-copy! v9 0)))
+(check-tail-exn wrong-number-of-arguments-exception? (lambda () (u8vector-copy! v9 0 v9 0 0 0)))
+
 (check-tail-exn range-exception? (lambda () (make-u8vector (expt 2 64))))
